#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

#include <eccodes.h>

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 74), // bufr_keys_iterator *()(grib_handle *, unsigned long)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 388), // grib_handle *
/*  2 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 10), // unsigned long
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  4 */ _CFFI_OP(_CFFI_OP_FUNCTION, 47), // char *()(bufr_keys_iterator const *)
/*  5 */ _CFFI_OP(_CFFI_OP_POINTER, 380), // bufr_keys_iterator const *
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION, 47), // char *()(grib_context const *)
/*  8 */ _CFFI_OP(_CFFI_OP_POINTER, 387), // grib_context const *
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 10 */ _CFFI_OP(_CFFI_OP_FUNCTION, 31), // char const *()(grib_keys_iterator const *)
/* 11 */ _CFFI_OP(_CFFI_OP_POINTER, 391), // grib_keys_iterator const *
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 13 */ _CFFI_OP(_CFFI_OP_FUNCTION, 31), // char const *()(int)
/* 14 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 15 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 16 */ _CFFI_OP(_CFFI_OP_FUNCTION, 19), // grib_context *()(void)
/* 17 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 18 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // grib_handle *()(grib_context *, FILE *, ProductKind, int *)
/* 19 */ _CFFI_OP(_CFFI_OP_POINTER, 387), // grib_context *
/* 20 */ _CFFI_OP(_CFFI_OP_POINTER, 379), // FILE *
/* 21 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // ProductKind
/* 22 */ _CFFI_OP(_CFFI_OP_POINTER, 14), // int *
/* 23 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // grib_handle *()(grib_context *, FILE *, int *)
/* 25 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 26 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 27 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 28 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 29 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // grib_handle *()(grib_context *, char const *)
/* 30 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 31 */ _CFFI_OP(_CFFI_OP_POINTER, 381), // char const *
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // grib_handle *()(grib_context *, void const *, size_t)
/* 34 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 35 */ _CFFI_OP(_CFFI_OP_POINTER, 395), // void const *
/* 36 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/* 37 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 38 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // grib_handle *()(grib_handle const *)
/* 39 */ _CFFI_OP(_CFFI_OP_POINTER, 388), // grib_handle const *
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 41 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // grib_handle *()(grib_index *, int *)
/* 42 */ _CFFI_OP(_CFFI_OP_POINTER, 389), // grib_index *
/* 43 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 44 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 45 */ _CFFI_OP(_CFFI_OP_FUNCTION, 42), // grib_index *()(grib_context *, char *, char const *, int *)
/* 46 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 47 */ _CFFI_OP(_CFFI_OP_POINTER, 381), // char *
/* 48 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 49 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 50 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 51 */ _CFFI_OP(_CFFI_OP_FUNCTION, 42), // grib_index *()(grib_context *, char const *, int *)
/* 52 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 53 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 54 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 55 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 56 */ _CFFI_OP(_CFFI_OP_FUNCTION, 314), // grib_iterator *()(grib_handle const *, unsigned long, int *)
/* 57 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 58 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 10),
/* 59 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 60 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 61 */ _CFFI_OP(_CFFI_OP_FUNCTION, 323), // grib_keys_iterator *()(grib_handle *, unsigned long, char const *)
/* 62 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 63 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 10),
/* 64 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 65 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 66 */ _CFFI_OP(_CFFI_OP_FUNCTION, 171), // grib_multi_handle *()(grib_context *)
/* 67 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 68 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 69 */ _CFFI_OP(_CFFI_OP_FUNCTION, 337), // grib_nearest *()(grib_handle const *, int *)
/* 70 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 71 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 72 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 73 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(bufr_keys_iterator *)
/* 74 */ _CFFI_OP(_CFFI_OP_POINTER, 380), // bufr_keys_iterator *
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 76 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(char const *, char *, int, int, grib_values *, int *)
/* 77 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 78 */ _CFFI_OP(_CFFI_OP_NOOP, 47),
/* 79 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 80 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 81 */ _CFFI_OP(_CFFI_OP_POINTER, 394), // grib_values *
/* 82 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 83 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 84 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_context *, FILE *, int *)
/* 85 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 86 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 87 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 88 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 89 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_context *, char const *, ProductKind, long * *, int *, int)
/* 90 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 91 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 92 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 93 */ _CFFI_OP(_CFFI_OP_POINTER, 227), // long * *
/* 94 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 95 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 96 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 97 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_context *, char const *, ProductKind, long * *, size_t * *, int *, int)
/* 98 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 99 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 100 */ _CFFI_OP(_CFFI_OP_NOOP, 21),
/* 101 */ _CFFI_OP(_CFFI_OP_NOOP, 93),
/* 102 */ _CFFI_OP(_CFFI_OP_POINTER, 130), // size_t * *
/* 103 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 104 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 105 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 106 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_context *, char const *, codes_bufr_header * *, int *, int)
/* 107 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 108 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 109 */ _CFFI_OP(_CFFI_OP_POINTER, 384), // codes_bufr_header * *
/* 110 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 111 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 112 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 113 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *)
/* 114 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 115 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 116 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *)
/* 117 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 118 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 119 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 120 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *, char const * *, size_t)
/* 121 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 122 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 123 */ _CFFI_OP(_CFFI_OP_POINTER, 31), // char const * *
/* 124 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 125 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 126 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *, char const *, size_t *)
/* 127 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 128 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 129 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 130 */ _CFFI_OP(_CFFI_OP_POINTER, 36), // size_t *
/* 131 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 132 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *, double const *, size_t)
/* 133 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 134 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 135 */ _CFFI_OP(_CFFI_OP_POINTER, 141), // double const *
/* 136 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 137 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 138 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *, double)
/* 139 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 140 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 141 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 142 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 143 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *, grib_handle *)
/* 144 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 145 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 146 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 147 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 148 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *, long const *, size_t)
/* 149 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 150 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 151 */ _CFFI_OP(_CFFI_OP_POINTER, 157), // long const *
/* 152 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 153 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 154 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, char const *, long)
/* 155 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 156 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 157 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9), // long
/* 158 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 159 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, grib_handle *)
/* 160 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 161 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 162 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 163 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, grib_values *, size_t)
/* 164 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 165 */ _CFFI_OP(_CFFI_OP_NOOP, 81),
/* 166 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 167 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 168 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle *, int, grib_multi_handle *)
/* 169 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 170 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 171 */ _CFFI_OP(_CFFI_OP_POINTER, 392), // grib_multi_handle *
/* 172 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 173 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *)
/* 174 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 175 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 176 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 177 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, char * *, size_t *)
/* 178 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 179 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 180 */ _CFFI_OP(_CFFI_OP_POINTER, 47), // char * *
/* 181 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 182 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 183 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, char *, size_t *)
/* 184 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 185 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 186 */ _CFFI_OP(_CFFI_OP_NOOP, 47),
/* 187 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 188 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 189 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, double *)
/* 190 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 191 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 192 */ _CFFI_OP(_CFFI_OP_POINTER, 141), // double *
/* 193 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 194 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, double *, size_t *)
/* 195 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 196 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 197 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 198 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 199 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 200 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, float *, size_t *)
/* 201 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 202 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 203 */ _CFFI_OP(_CFFI_OP_POINTER, 386), // float *
/* 204 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 205 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 206 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, int *)
/* 207 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 208 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 209 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 210 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 211 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, int const *, long, double *)
/* 212 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 213 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 214 */ _CFFI_OP(_CFFI_OP_POINTER, 14), // int const *
/* 215 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 216 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 217 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 218 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, int, double *)
/* 219 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 220 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 221 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 222 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 223 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 224 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, long *)
/* 225 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 226 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 227 */ _CFFI_OP(_CFFI_OP_POINTER, 157), // long *
/* 228 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 229 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, long *, size_t *)
/* 230 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 231 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 232 */ _CFFI_OP(_CFFI_OP_NOOP, 227),
/* 233 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 234 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 235 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, char const *, size_t *)
/* 236 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 237 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 238 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 239 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 240 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, double *, double *, double *)
/* 241 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 242 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 243 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 244 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 245 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 246 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, int, double const *, double const *, long, double *, double *, double *, double *, int *)
/* 247 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 248 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 249 */ _CFFI_OP(_CFFI_OP_NOOP, 135),
/* 250 */ _CFFI_OP(_CFFI_OP_NOOP, 135),
/* 251 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 252 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 253 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 254 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 255 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 256 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 257 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 258 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, long *)
/* 259 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 260 */ _CFFI_OP(_CFFI_OP_NOOP, 227),
/* 261 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 262 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, size_t *)
/* 263 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 264 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 265 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 266 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_handle const *, void const * *, size_t *)
/* 267 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 268 */ _CFFI_OP(_CFFI_OP_POINTER, 35), // void const * *
/* 269 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 270 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 271 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index *, char const *)
/* 272 */ _CFFI_OP(_CFFI_OP_NOOP, 42),
/* 273 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 274 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 275 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index *, char const *, char *)
/* 276 */ _CFFI_OP(_CFFI_OP_NOOP, 42),
/* 277 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 278 */ _CFFI_OP(_CFFI_OP_NOOP, 47),
/* 279 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 280 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index *, char const *, double)
/* 281 */ _CFFI_OP(_CFFI_OP_NOOP, 42),
/* 282 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 283 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 284 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 285 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index *, char const *, long)
/* 286 */ _CFFI_OP(_CFFI_OP_NOOP, 42),
/* 287 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 288 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 289 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 290 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index const *, char const *, char * *, size_t *)
/* 291 */ _CFFI_OP(_CFFI_OP_POINTER, 389), // grib_index const *
/* 292 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 293 */ _CFFI_OP(_CFFI_OP_NOOP, 180),
/* 294 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 295 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 296 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index const *, char const *, double *, size_t *)
/* 297 */ _CFFI_OP(_CFFI_OP_NOOP, 291),
/* 298 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 299 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 300 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 301 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 302 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index const *, char const *, long *, size_t *)
/* 303 */ _CFFI_OP(_CFFI_OP_NOOP, 291),
/* 304 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 305 */ _CFFI_OP(_CFFI_OP_NOOP, 227),
/* 306 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 307 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 308 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_index const *, char const *, size_t *)
/* 309 */ _CFFI_OP(_CFFI_OP_NOOP, 291),
/* 310 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 311 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 312 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 313 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_iterator *)
/* 314 */ _CFFI_OP(_CFFI_OP_POINTER, 390), // grib_iterator *
/* 315 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 316 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_iterator *, double *, double *, double *)
/* 317 */ _CFFI_OP(_CFFI_OP_NOOP, 314),
/* 318 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 319 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 320 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 321 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 322 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_keys_iterator *)
/* 323 */ _CFFI_OP(_CFFI_OP_POINTER, 391), // grib_keys_iterator *
/* 324 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 325 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_keys_iterator *, unsigned long)
/* 326 */ _CFFI_OP(_CFFI_OP_NOOP, 323),
/* 327 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 10),
/* 328 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 329 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_multi_handle *)
/* 330 */ _CFFI_OP(_CFFI_OP_NOOP, 171),
/* 331 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 332 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_multi_handle *, FILE *)
/* 333 */ _CFFI_OP(_CFFI_OP_NOOP, 171),
/* 334 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 335 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 336 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_nearest *)
/* 337 */ _CFFI_OP(_CFFI_OP_POINTER, 393), // grib_nearest *
/* 338 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 339 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(grib_nearest *, grib_handle const *, double, double, unsigned long, double *, double *, double *, double *, int *, size_t *)
/* 340 */ _CFFI_OP(_CFFI_OP_NOOP, 337),
/* 341 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 342 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 343 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 344 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 10),
/* 345 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 346 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 347 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 348 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 349 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 350 */ _CFFI_OP(_CFFI_OP_NOOP, 130),
/* 351 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 352 */ _CFFI_OP(_CFFI_OP_FUNCTION, 14), // int()(long, double *)
/* 353 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 354 */ _CFFI_OP(_CFFI_OP_NOOP, 192),
/* 355 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 356 */ _CFFI_OP(_CFFI_OP_FUNCTION, 157), // long()(void)
/* 357 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 358 */ _CFFI_OP(_CFFI_OP_FUNCTION, 395), // void()(grib_context *)
/* 359 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 360 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 361 */ _CFFI_OP(_CFFI_OP_FUNCTION, 395), // void()(grib_context *, FILE *)
/* 362 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 363 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 364 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 365 */ _CFFI_OP(_CFFI_OP_FUNCTION, 395), // void()(grib_context *, char const *)
/* 366 */ _CFFI_OP(_CFFI_OP_NOOP, 19),
/* 367 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 368 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 369 */ _CFFI_OP(_CFFI_OP_FUNCTION, 395), // void()(grib_handle const *, FILE *, char const *, unsigned long, void *)
/* 370 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 371 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 372 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 373 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 10),
/* 374 */ _CFFI_OP(_CFFI_OP_POINTER, 395), // void *
/* 375 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 376 */ _CFFI_OP(_CFFI_OP_FUNCTION, 395), // void()(grib_index *)
/* 377 */ _CFFI_OP(_CFFI_OP_NOOP, 42),
/* 378 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 379 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // FILE
/* 380 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // bufr_keys_iterator
/* 381 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 382 */ _CFFI_OP(_CFFI_OP_ARRAY, 381), // char[9]
/* 383 */ (_cffi_opcode_t)(9),
/* 384 */ _CFFI_OP(_CFFI_OP_POINTER, 385), // codes_bufr_header *
/* 385 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // codes_bufr_header
/* 386 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/* 387 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // grib_context
/* 388 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // grib_handle
/* 389 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // grib_index
/* 390 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // grib_iterator
/* 391 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // grib_keys_iterator
/* 392 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 8), // grib_multi_handle
/* 393 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 9), // grib_nearest
/* 394 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 10), // grib_values
/* 395 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const_PRODUCT_ANY(unsigned long long *o)
{
  int n = (PRODUCT_ANY) <= 0;
  *o = (unsigned long long)((PRODUCT_ANY) | 0);  /* check that PRODUCT_ANY is an integer */
  return n;
}

static int _cffi_const_PRODUCT_GRIB(unsigned long long *o)
{
  int n = (PRODUCT_GRIB) <= 0;
  *o = (unsigned long long)((PRODUCT_GRIB) | 0);  /* check that PRODUCT_GRIB is an integer */
  return n;
}

static int _cffi_const_PRODUCT_BUFR(unsigned long long *o)
{
  int n = (PRODUCT_BUFR) <= 0;
  *o = (unsigned long long)((PRODUCT_BUFR) | 0);  /* check that PRODUCT_BUFR is an integer */
  return n;
}

static int _cffi_const_PRODUCT_METAR(unsigned long long *o)
{
  int n = (PRODUCT_METAR) <= 0;
  *o = (unsigned long long)((PRODUCT_METAR) | 0);  /* check that PRODUCT_METAR is an integer */
  return n;
}

static int _cffi_const_PRODUCT_GTS(unsigned long long *o)
{
  int n = (PRODUCT_GTS) <= 0;
  *o = (unsigned long long)((PRODUCT_GTS) | 0);  /* check that PRODUCT_GTS is an integer */
  return n;
}

static int _cffi_const_PRODUCT_TAF(unsigned long long *o)
{
  int n = (PRODUCT_TAF) <= 0;
  *o = (unsigned long long)((PRODUCT_TAF) | 0);  /* check that PRODUCT_TAF is an integer */
  return n;
}

static int _cffi_d_codes_bufr_copy_data(grib_handle * x0, grib_handle * x1)
{
  return codes_bufr_copy_data(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_copy_data(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  grib_handle * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "codes_bufr_copy_data", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_copy_data(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_copy_data _cffi_d_codes_bufr_copy_data
#endif

static int _cffi_d_codes_bufr_extract_headers_malloc(grib_context * x0, char const * x1, codes_bufr_header * * x2, int * x3, int x4)
{
  return codes_bufr_extract_headers_malloc(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_extract_headers_malloc(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  codes_bufr_header * * x2;
  int * x3;
  int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "codes_bufr_extract_headers_malloc", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(109), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (codes_bufr_header * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(109), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_extract_headers_malloc(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_extract_headers_malloc _cffi_d_codes_bufr_extract_headers_malloc
#endif

static grib_handle * _cffi_d_codes_bufr_handle_new_from_samples(grib_context * x0, char const * x1)
{
  return codes_bufr_handle_new_from_samples(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_handle_new_from_samples(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "codes_bufr_handle_new_from_samples", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_handle_new_from_samples(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_handle_new_from_samples _cffi_d_codes_bufr_handle_new_from_samples
#endif

static int _cffi_d_codes_bufr_key_is_coordinate(grib_handle const * x0, char const * x1, int * x2)
{
  return codes_bufr_key_is_coordinate(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_key_is_coordinate(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "codes_bufr_key_is_coordinate", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_key_is_coordinate(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_key_is_coordinate _cffi_d_codes_bufr_key_is_coordinate
#endif

static int _cffi_d_codes_bufr_key_is_header(grib_handle const * x0, char const * x1, int * x2)
{
  return codes_bufr_key_is_header(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_key_is_header(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "codes_bufr_key_is_header", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_key_is_header(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_key_is_header _cffi_d_codes_bufr_key_is_header
#endif

static int _cffi_d_codes_bufr_keys_iterator_delete(bufr_keys_iterator * x0)
{
  return codes_bufr_keys_iterator_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_keys_iterator_delete(PyObject *self, PyObject *arg0)
{
  bufr_keys_iterator * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(74), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (bufr_keys_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(74), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_keys_iterator_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_keys_iterator_delete _cffi_d_codes_bufr_keys_iterator_delete
#endif

static char * _cffi_d_codes_bufr_keys_iterator_get_name(bufr_keys_iterator const * x0)
{
  return codes_bufr_keys_iterator_get_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_keys_iterator_get_name(PyObject *self, PyObject *arg0)
{
  bufr_keys_iterator const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (bufr_keys_iterator const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_keys_iterator_get_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(47));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_keys_iterator_get_name _cffi_d_codes_bufr_keys_iterator_get_name
#endif

static bufr_keys_iterator * _cffi_d_codes_bufr_keys_iterator_new(grib_handle * x0, unsigned long x1)
{
  return codes_bufr_keys_iterator_new(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_keys_iterator_new(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  unsigned long x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  bufr_keys_iterator * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "codes_bufr_keys_iterator_new", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned long);
  if (x1 == (unsigned long)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_keys_iterator_new(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(74));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_keys_iterator_new _cffi_d_codes_bufr_keys_iterator_new
#endif

static int _cffi_d_codes_bufr_keys_iterator_next(bufr_keys_iterator * x0)
{
  return codes_bufr_keys_iterator_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_keys_iterator_next(PyObject *self, PyObject *arg0)
{
  bufr_keys_iterator * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(74), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (bufr_keys_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(74), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_keys_iterator_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_keys_iterator_next _cffi_d_codes_bufr_keys_iterator_next
#endif

static int _cffi_d_codes_bufr_keys_iterator_rewind(bufr_keys_iterator * x0)
{
  return codes_bufr_keys_iterator_rewind(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_keys_iterator_rewind(PyObject *self, PyObject *arg0)
{
  bufr_keys_iterator * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(74), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (bufr_keys_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(74), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_bufr_keys_iterator_rewind(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_bufr_keys_iterator_rewind _cffi_d_codes_bufr_keys_iterator_rewind
#endif

static void _cffi_d_codes_bufr_multi_element_constant_arrays_off(grib_context * x0)
{
  codes_bufr_multi_element_constant_arrays_off(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_multi_element_constant_arrays_off(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { codes_bufr_multi_element_constant_arrays_off(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_codes_bufr_multi_element_constant_arrays_off _cffi_d_codes_bufr_multi_element_constant_arrays_off
#endif

static void _cffi_d_codes_bufr_multi_element_constant_arrays_on(grib_context * x0)
{
  codes_bufr_multi_element_constant_arrays_on(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_bufr_multi_element_constant_arrays_on(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { codes_bufr_multi_element_constant_arrays_on(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_codes_bufr_multi_element_constant_arrays_on _cffi_d_codes_bufr_multi_element_constant_arrays_on
#endif

static char * _cffi_d_codes_definition_path(grib_context const * x0)
{
  return codes_definition_path(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_definition_path(PyObject *self, PyObject *arg0)
{
  grib_context const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_definition_path(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(47));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_definition_path _cffi_d_codes_definition_path
#endif

static int _cffi_d_codes_extract_offsets_malloc(grib_context * x0, char const * x1, ProductKind x2, long * * x3, int * x4, int x5)
{
  return codes_extract_offsets_malloc(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_extract_offsets_malloc(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  ProductKind x2;
  long * * x3;
  int * x4;
  int x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "codes_extract_offsets_malloc", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(21), arg2) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(93), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (long * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(93), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = _cffi_to_c_int(arg5, int);
  if (x5 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_extract_offsets_malloc(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_extract_offsets_malloc _cffi_d_codes_extract_offsets_malloc
#endif

static int _cffi_d_codes_extract_offsets_sizes_malloc(grib_context * x0, char const * x1, ProductKind x2, long * * x3, size_t * * x4, int * x5, int x6)
{
  return codes_extract_offsets_sizes_malloc(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_extract_offsets_sizes_malloc(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  ProductKind x2;
  long * * x3;
  size_t * * x4;
  int * x5;
  int x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "codes_extract_offsets_sizes_malloc", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(21), arg2) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(93), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (long * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(93), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(102), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (size_t * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(102), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_extract_offsets_sizes_malloc(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_extract_offsets_sizes_malloc _cffi_d_codes_extract_offsets_sizes_malloc
#endif

static grib_handle * _cffi_d_codes_handle_new_from_file(grib_context * x0, FILE * x1, ProductKind x2, int * x3)
{
  return codes_handle_new_from_file(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_handle_new_from_file(PyObject *self, PyObject *args)
{
  grib_context * x0;
  FILE * x1;
  ProductKind x2;
  int * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "codes_handle_new_from_file", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(21), arg2) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_handle_new_from_file(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_handle_new_from_file _cffi_d_codes_handle_new_from_file
#endif

static grib_handle * _cffi_d_codes_handle_new_from_samples(grib_context * x0, char const * x1)
{
  return codes_handle_new_from_samples(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_handle_new_from_samples(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "codes_handle_new_from_samples", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_handle_new_from_samples(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_handle_new_from_samples _cffi_d_codes_handle_new_from_samples
#endif

static char * _cffi_d_codes_samples_path(grib_context const * x0)
{
  return codes_samples_path(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_codes_samples_path(PyObject *self, PyObject *arg0)
{
  grib_context const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = codes_samples_path(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(47));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_codes_samples_path _cffi_d_codes_samples_path
#endif

static void _cffi_d_grib_context_delete(grib_context * x0)
{
  grib_context_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_context_delete(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_context_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_context_delete _cffi_d_grib_context_delete
#endif

static grib_context * _cffi_d_grib_context_get_default(void)
{
  return grib_context_get_default();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_context_get_default(PyObject *self, PyObject *noarg)
{
  grib_context * result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_context_get_default(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(19));
  return pyresult;
}
#else
#  define _cffi_f_grib_context_get_default _cffi_d_grib_context_get_default
#endif

static void _cffi_d_grib_context_set_definitions_path(grib_context * x0, char const * x1)
{
  grib_context_set_definitions_path(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_context_set_definitions_path(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_context_set_definitions_path", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_context_set_definitions_path(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_context_set_definitions_path _cffi_d_grib_context_set_definitions_path
#endif

static void _cffi_d_grib_context_set_samples_path(grib_context * x0, char const * x1)
{
  grib_context_set_samples_path(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_context_set_samples_path(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_context_set_samples_path", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_context_set_samples_path(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_context_set_samples_path _cffi_d_grib_context_set_samples_path
#endif

static int _cffi_d_grib_copy_namespace(grib_handle * x0, char const * x1, grib_handle * x2)
{
  return grib_copy_namespace(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_copy_namespace(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  grib_handle * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_copy_namespace", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_copy_namespace(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_copy_namespace _cffi_d_grib_copy_namespace
#endif

static int _cffi_d_grib_count_in_file(grib_context * x0, FILE * x1, int * x2)
{
  return grib_count_in_file(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_count_in_file(PyObject *self, PyObject *args)
{
  grib_context * x0;
  FILE * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_count_in_file", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_count_in_file(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_count_in_file _cffi_d_grib_count_in_file
#endif

static char * _cffi_d_grib_definition_path(grib_context const * x0)
{
  return grib_definition_path(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_definition_path(PyObject *self, PyObject *arg0)
{
  grib_context const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_definition_path(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(47));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_definition_path _cffi_d_grib_definition_path
#endif

static void _cffi_d_grib_dump_content(grib_handle const * x0, FILE * x1, char const * x2, unsigned long x3, void * x4)
{
  grib_dump_content(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_dump_content(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  FILE * x1;
  char const * x2;
  unsigned long x3;
  void * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "grib_dump_content", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, unsigned long);
  if (x3 == (unsigned long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(374), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(374), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_dump_content(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_dump_content _cffi_d_grib_dump_content
#endif

static long _cffi_d_grib_get_api_version(void)
{
  return grib_get_api_version();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_api_version(PyObject *self, PyObject *noarg)
{
  long result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_api_version(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, long);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_api_version _cffi_d_grib_get_api_version
#endif

static int _cffi_d_grib_get_data(grib_handle const * x0, double * x1, double * x2, double * x3)
{
  return grib_get_data(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_data(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  double * x1;
  double * x2;
  double * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_get_data", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_data(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_data _cffi_d_grib_get_data
#endif

static int _cffi_d_grib_get_double(grib_handle const * x0, char const * x1, double * x2)
{
  return grib_get_double(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_double(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  double * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_get_double", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_double(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_double _cffi_d_grib_get_double
#endif

static int _cffi_d_grib_get_double_array(grib_handle const * x0, char const * x1, double * x2, size_t * x3)
{
  return grib_get_double_array(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_double_array(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  double * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_get_double_array", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_double_array(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_double_array _cffi_d_grib_get_double_array
#endif

static int _cffi_d_grib_get_double_element(grib_handle const * x0, char const * x1, int x2, double * x3)
{
  return grib_get_double_element(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_double_element(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  int x2;
  double * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_get_double_element", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_double_element(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_double_element _cffi_d_grib_get_double_element
#endif

static int _cffi_d_grib_get_double_elements(grib_handle const * x0, char const * x1, int const * x2, long x3, double * x4)
{
  return grib_get_double_elements(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_double_elements(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  int const * x2;
  long x3;
  double * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "grib_get_double_elements", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(214), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(214), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, long);
  if (x3 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_double_elements(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_double_elements _cffi_d_grib_get_double_elements
#endif

static char const * _cffi_d_grib_get_error_message(int x0)
{
  return grib_get_error_message(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_error_message(PyObject *self, PyObject *arg0)
{
  int x0;
  char const * result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_error_message(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(31));
  return pyresult;
}
#else
#  define _cffi_f_grib_get_error_message _cffi_d_grib_get_error_message
#endif

static int _cffi_d_grib_get_float_array(grib_handle const * x0, char const * x1, float * x2, size_t * x3)
{
  return grib_get_float_array(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_float_array(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  float * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_get_float_array", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(203), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (float *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(203), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_float_array(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_float_array _cffi_d_grib_get_float_array
#endif

static int _cffi_d_grib_get_gaussian_latitudes(long x0, double * x1)
{
  return grib_get_gaussian_latitudes(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_gaussian_latitudes(PyObject *self, PyObject *args)
{
  long x0;
  double * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_get_gaussian_latitudes", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, long);
  if (x0 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_gaussian_latitudes(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_gaussian_latitudes _cffi_d_grib_get_gaussian_latitudes
#endif

static int _cffi_d_grib_get_length(grib_handle const * x0, char const * x1, size_t * x2)
{
  return grib_get_length(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_length(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  size_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_get_length", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_length(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_length _cffi_d_grib_get_length
#endif

static int _cffi_d_grib_get_long(grib_handle const * x0, char const * x1, long * x2)
{
  return grib_get_long(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_long(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  long * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_get_long", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(227), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(227), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_long(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_long _cffi_d_grib_get_long
#endif

static int _cffi_d_grib_get_long_array(grib_handle const * x0, char const * x1, long * x2, size_t * x3)
{
  return grib_get_long_array(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_long_array(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  long * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_get_long_array", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(227), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(227), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_long_array(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_long_array _cffi_d_grib_get_long_array
#endif

static int _cffi_d_grib_get_message(grib_handle const * x0, void const * * x1, size_t * x2)
{
  return grib_get_message(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_message(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  void const * * x1;
  size_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_get_message", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(268), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void const * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(268), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_message(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_message _cffi_d_grib_get_message
#endif

static int _cffi_d_grib_get_message_offset(grib_handle const * x0, long * x1)
{
  return grib_get_message_offset(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_message_offset(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  long * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_get_message_offset", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(227), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(227), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_message_offset(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_message_offset _cffi_d_grib_get_message_offset
#endif

static int _cffi_d_grib_get_message_size(grib_handle const * x0, size_t * x1)
{
  return grib_get_message_size(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_message_size(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  size_t * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_get_message_size", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_message_size(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_message_size _cffi_d_grib_get_message_size
#endif

static int _cffi_d_grib_get_native_type(grib_handle const * x0, char const * x1, int * x2)
{
  return grib_get_native_type(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_native_type(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_get_native_type", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_native_type(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_native_type _cffi_d_grib_get_native_type
#endif

static int _cffi_d_grib_get_size(grib_handle const * x0, char const * x1, size_t * x2)
{
  return grib_get_size(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_size(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  size_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_get_size", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_size(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_size _cffi_d_grib_get_size
#endif

static int _cffi_d_grib_get_string(grib_handle const * x0, char const * x1, char * x2, size_t * x3)
{
  return grib_get_string(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_string(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  char * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_get_string", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(47), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(47), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_string(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_string _cffi_d_grib_get_string
#endif

static int _cffi_d_grib_get_string_array(grib_handle const * x0, char const * x1, char * * x2, size_t * x3)
{
  return grib_get_string_array(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_get_string_array(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  char * * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_get_string_array", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(180), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(180), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_get_string_array(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_get_string_array _cffi_d_grib_get_string_array
#endif

static void _cffi_d_grib_gribex_mode_off(grib_context * x0)
{
  grib_gribex_mode_off(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_gribex_mode_off(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_gribex_mode_off(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_gribex_mode_off _cffi_d_grib_gribex_mode_off
#endif

static void _cffi_d_grib_gribex_mode_on(grib_context * x0)
{
  grib_gribex_mode_on(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_gribex_mode_on(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_gribex_mode_on(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_gribex_mode_on _cffi_d_grib_gribex_mode_on
#endif

static void _cffi_d_grib_gts_header_off(grib_context * x0)
{
  grib_gts_header_off(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_gts_header_off(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_gts_header_off(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_gts_header_off _cffi_d_grib_gts_header_off
#endif

static void _cffi_d_grib_gts_header_on(grib_context * x0)
{
  grib_gts_header_on(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_gts_header_on(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_gts_header_on(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_gts_header_on _cffi_d_grib_gts_header_on
#endif

static grib_handle * _cffi_d_grib_handle_clone(grib_handle const * x0)
{
  return grib_handle_clone(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_handle_clone(PyObject *self, PyObject *arg0)
{
  grib_handle const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_handle_clone(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_handle_clone _cffi_d_grib_handle_clone
#endif

static grib_handle * _cffi_d_grib_handle_clone_headers_only(grib_handle const * x0)
{
  return grib_handle_clone_headers_only(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_handle_clone_headers_only(PyObject *self, PyObject *arg0)
{
  grib_handle const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_handle_clone_headers_only(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_handle_clone_headers_only _cffi_d_grib_handle_clone_headers_only
#endif

static int _cffi_d_grib_handle_delete(grib_handle * x0)
{
  return grib_handle_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_handle_delete(PyObject *self, PyObject *arg0)
{
  grib_handle * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_handle_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_handle_delete _cffi_d_grib_handle_delete
#endif

static grib_handle * _cffi_d_grib_handle_new_from_file(grib_context * x0, FILE * x1, int * x2)
{
  return grib_handle_new_from_file(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_handle_new_from_file(PyObject *self, PyObject *args)
{
  grib_context * x0;
  FILE * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_handle_new_from_file", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_handle_new_from_file(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_handle_new_from_file _cffi_d_grib_handle_new_from_file
#endif

static grib_handle * _cffi_d_grib_handle_new_from_index(grib_index * x0, int * x1)
{
  return grib_handle_new_from_index(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_handle_new_from_index(PyObject *self, PyObject *args)
{
  grib_index * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_handle_new_from_index", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(42), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(42), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_handle_new_from_index(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_handle_new_from_index _cffi_d_grib_handle_new_from_index
#endif

static grib_handle * _cffi_d_grib_handle_new_from_message_copy(grib_context * x0, void const * x1, size_t x2)
{
  return grib_handle_new_from_message_copy(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_handle_new_from_message_copy(PyObject *self, PyObject *args)
{
  grib_context * x0;
  void const * x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_handle_new_from_message_copy", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(35), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(35), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_handle_new_from_message_copy(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_handle_new_from_message_copy _cffi_d_grib_handle_new_from_message_copy
#endif

static grib_handle * _cffi_d_grib_handle_new_from_samples(grib_context * x0, char const * x1)
{
  return grib_handle_new_from_samples(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_handle_new_from_samples(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_handle * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_handle_new_from_samples", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_handle_new_from_samples(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_handle_new_from_samples _cffi_d_grib_handle_new_from_samples
#endif

static int _cffi_d_grib_index_add_file(grib_index * x0, char const * x1)
{
  return grib_index_add_file(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_add_file(PyObject *self, PyObject *args)
{
  grib_index * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_index_add_file", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(42), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(42), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_add_file(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_add_file _cffi_d_grib_index_add_file
#endif

static void _cffi_d_grib_index_delete(grib_index * x0)
{
  grib_index_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_delete(PyObject *self, PyObject *arg0)
{
  grib_index * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(42), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(42), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_index_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_index_delete _cffi_d_grib_index_delete
#endif

static int _cffi_d_grib_index_get_double(grib_index const * x0, char const * x1, double * x2, size_t * x3)
{
  return grib_index_get_double(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_get_double(PyObject *self, PyObject *args)
{
  grib_index const * x0;
  char const * x1;
  double * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_index_get_double", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(291), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(291), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_get_double(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_get_double _cffi_d_grib_index_get_double
#endif

static int _cffi_d_grib_index_get_long(grib_index const * x0, char const * x1, long * x2, size_t * x3)
{
  return grib_index_get_long(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_get_long(PyObject *self, PyObject *args)
{
  grib_index const * x0;
  char const * x1;
  long * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_index_get_long", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(291), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(291), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(227), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(227), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_get_long(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_get_long _cffi_d_grib_index_get_long
#endif

static int _cffi_d_grib_index_get_size(grib_index const * x0, char const * x1, size_t * x2)
{
  return grib_index_get_size(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_get_size(PyObject *self, PyObject *args)
{
  grib_index const * x0;
  char const * x1;
  size_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_index_get_size", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(291), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(291), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_get_size(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_get_size _cffi_d_grib_index_get_size
#endif

static int _cffi_d_grib_index_get_string(grib_index const * x0, char const * x1, char * * x2, size_t * x3)
{
  return grib_index_get_string(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_get_string(PyObject *self, PyObject *args)
{
  grib_index const * x0;
  char const * x1;
  char * * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_index_get_string", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(291), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(291), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(180), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(180), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_get_string(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_get_string _cffi_d_grib_index_get_string
#endif

static grib_index * _cffi_d_grib_index_new_from_file(grib_context * x0, char * x1, char const * x2, int * x3)
{
  return grib_index_new_from_file(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_new_from_file(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char * x1;
  char const * x2;
  int * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_index * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_index_new_from_file", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(47), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(47), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_new_from_file(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(42));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_new_from_file _cffi_d_grib_index_new_from_file
#endif

static grib_index * _cffi_d_grib_index_read(grib_context * x0, char const * x1, int * x2)
{
  return grib_index_read(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_read(PyObject *self, PyObject *args)
{
  grib_context * x0;
  char const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_index * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_index_read", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_read(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(42));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_read _cffi_d_grib_index_read
#endif

static int _cffi_d_grib_index_select_double(grib_index * x0, char const * x1, double x2)
{
  return grib_index_select_double(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_select_double(PyObject *self, PyObject *args)
{
  grib_index * x0;
  char const * x1;
  double x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_index_select_double", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(42), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(42), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (double)_cffi_to_c_double(arg2);
  if (x2 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_select_double(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_select_double _cffi_d_grib_index_select_double
#endif

static int _cffi_d_grib_index_select_long(grib_index * x0, char const * x1, long x2)
{
  return grib_index_select_long(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_select_long(PyObject *self, PyObject *args)
{
  grib_index * x0;
  char const * x1;
  long x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_index_select_long", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(42), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(42), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_select_long(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_select_long _cffi_d_grib_index_select_long
#endif

static int _cffi_d_grib_index_select_string(grib_index * x0, char const * x1, char * x2)
{
  return grib_index_select_string(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_select_string(PyObject *self, PyObject *args)
{
  grib_index * x0;
  char const * x1;
  char * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_index_select_string", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(42), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(42), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(47), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(47), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_select_string(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_select_string _cffi_d_grib_index_select_string
#endif

static int _cffi_d_grib_index_write(grib_index * x0, char const * x1)
{
  return grib_index_write(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_index_write(PyObject *self, PyObject *args)
{
  grib_index * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_index_write", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(42), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_index *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(42), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_index_write(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_index_write _cffi_d_grib_index_write
#endif

static int _cffi_d_grib_is_defined(grib_handle const * x0, char const * x1)
{
  return grib_is_defined(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_is_defined(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_is_defined", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_is_defined(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_is_defined _cffi_d_grib_is_defined
#endif

static int _cffi_d_grib_is_missing(grib_handle const * x0, char const * x1, int * x2)
{
  return grib_is_missing(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_is_missing(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  char const * x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_is_missing", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_is_missing(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_is_missing _cffi_d_grib_is_missing
#endif

static int _cffi_d_grib_iterator_delete(grib_iterator * x0)
{
  return grib_iterator_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_iterator_delete(PyObject *self, PyObject *arg0)
{
  grib_iterator * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(314), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(314), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_iterator_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_iterator_delete _cffi_d_grib_iterator_delete
#endif

static grib_iterator * _cffi_d_grib_iterator_new(grib_handle const * x0, unsigned long x1, int * x2)
{
  return grib_iterator_new(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_iterator_new(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  unsigned long x1;
  int * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_iterator * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_iterator_new", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned long);
  if (x1 == (unsigned long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_iterator_new(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(314));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_iterator_new _cffi_d_grib_iterator_new
#endif

static int _cffi_d_grib_iterator_next(grib_iterator * x0, double * x1, double * x2, double * x3)
{
  return grib_iterator_next(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_iterator_next(PyObject *self, PyObject *args)
{
  grib_iterator * x0;
  double * x1;
  double * x2;
  double * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_iterator_next", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(314), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(314), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_iterator_next(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_iterator_next _cffi_d_grib_iterator_next
#endif

static int _cffi_d_grib_keys_iterator_delete(grib_keys_iterator * x0)
{
  return grib_keys_iterator_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_keys_iterator_delete(PyObject *self, PyObject *arg0)
{
  grib_keys_iterator * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(323), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_keys_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(323), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_keys_iterator_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_keys_iterator_delete _cffi_d_grib_keys_iterator_delete
#endif

static char const * _cffi_d_grib_keys_iterator_get_name(grib_keys_iterator const * x0)
{
  return grib_keys_iterator_get_name(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_keys_iterator_get_name(PyObject *self, PyObject *arg0)
{
  grib_keys_iterator const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_keys_iterator const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_keys_iterator_get_name(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(31));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_keys_iterator_get_name _cffi_d_grib_keys_iterator_get_name
#endif

static grib_keys_iterator * _cffi_d_grib_keys_iterator_new(grib_handle * x0, unsigned long x1, char const * x2)
{
  return grib_keys_iterator_new(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_keys_iterator_new(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  unsigned long x1;
  char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_keys_iterator * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_keys_iterator_new", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned long);
  if (x1 == (unsigned long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_keys_iterator_new(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(323));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_keys_iterator_new _cffi_d_grib_keys_iterator_new
#endif

static int _cffi_d_grib_keys_iterator_next(grib_keys_iterator * x0)
{
  return grib_keys_iterator_next(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_keys_iterator_next(PyObject *self, PyObject *arg0)
{
  grib_keys_iterator * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(323), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_keys_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(323), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_keys_iterator_next(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_keys_iterator_next _cffi_d_grib_keys_iterator_next
#endif

static int _cffi_d_grib_keys_iterator_rewind(grib_keys_iterator * x0)
{
  return grib_keys_iterator_rewind(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_keys_iterator_rewind(PyObject *self, PyObject *arg0)
{
  grib_keys_iterator * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(323), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_keys_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(323), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_keys_iterator_rewind(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_keys_iterator_rewind _cffi_d_grib_keys_iterator_rewind
#endif

static int _cffi_d_grib_keys_iterator_set_flags(grib_keys_iterator * x0, unsigned long x1)
{
  return grib_keys_iterator_set_flags(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_keys_iterator_set_flags(PyObject *self, PyObject *args)
{
  grib_keys_iterator * x0;
  unsigned long x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_keys_iterator_set_flags", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(323), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_keys_iterator *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(323), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned long);
  if (x1 == (unsigned long)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_keys_iterator_set_flags(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_keys_iterator_set_flags _cffi_d_grib_keys_iterator_set_flags
#endif

static int _cffi_d_grib_multi_handle_append(grib_handle * x0, int x1, grib_multi_handle * x2)
{
  return grib_multi_handle_append(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_multi_handle_append(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  int x1;
  grib_multi_handle * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_multi_handle_append", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(171), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (grib_multi_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(171), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_multi_handle_append(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_multi_handle_append _cffi_d_grib_multi_handle_append
#endif

static int _cffi_d_grib_multi_handle_delete(grib_multi_handle * x0)
{
  return grib_multi_handle_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_multi_handle_delete(PyObject *self, PyObject *arg0)
{
  grib_multi_handle * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(171), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_multi_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(171), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_multi_handle_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_multi_handle_delete _cffi_d_grib_multi_handle_delete
#endif

static grib_multi_handle * _cffi_d_grib_multi_handle_new(grib_context * x0)
{
  return grib_multi_handle_new(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_multi_handle_new(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_multi_handle * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_multi_handle_new(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(171));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_multi_handle_new _cffi_d_grib_multi_handle_new
#endif

static int _cffi_d_grib_multi_handle_write(grib_multi_handle * x0, FILE * x1)
{
  return grib_multi_handle_write(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_multi_handle_write(PyObject *self, PyObject *args)
{
  grib_multi_handle * x0;
  FILE * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_multi_handle_write", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(171), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_multi_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(171), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_multi_handle_write(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_multi_handle_write _cffi_d_grib_multi_handle_write
#endif

static void _cffi_d_grib_multi_support_off(grib_context * x0)
{
  grib_multi_support_off(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_multi_support_off(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_multi_support_off(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_multi_support_off _cffi_d_grib_multi_support_off
#endif

static void _cffi_d_grib_multi_support_on(grib_context * x0)
{
  grib_multi_support_on(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_multi_support_on(PyObject *self, PyObject *arg0)
{
  grib_context * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_multi_support_on(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_multi_support_on _cffi_d_grib_multi_support_on
#endif

static void _cffi_d_grib_multi_support_reset_file(grib_context * x0, FILE * x1)
{
  grib_multi_support_reset_file(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_multi_support_reset_file(PyObject *self, PyObject *args)
{
  grib_context * x0;
  FILE * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_multi_support_reset_file", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (FILE *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { grib_multi_support_reset_file(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_grib_multi_support_reset_file _cffi_d_grib_multi_support_reset_file
#endif

static int _cffi_d_grib_nearest_delete(grib_nearest * x0)
{
  return grib_nearest_delete(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_nearest_delete(PyObject *self, PyObject *arg0)
{
  grib_nearest * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(337), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_nearest *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(337), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_nearest_delete(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_nearest_delete _cffi_d_grib_nearest_delete
#endif

static int _cffi_d_grib_nearest_find(grib_nearest * x0, grib_handle const * x1, double x2, double x3, unsigned long x4, double * x5, double * x6, double * x7, double * x8, int * x9, size_t * x10)
{
  return grib_nearest_find(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_nearest_find(PyObject *self, PyObject *args)
{
  grib_nearest * x0;
  grib_handle const * x1;
  double x2;
  double x3;
  unsigned long x4;
  double * x5;
  double * x6;
  double * x7;
  double * x8;
  int * x9;
  size_t * x10;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;
  PyObject *arg9;
  PyObject *arg10;

  if (!PyArg_UnpackTuple(args, "grib_nearest_find", 11, 11, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9, &arg10))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(337), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_nearest *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(337), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (double)_cffi_to_c_double(arg2);
  if (x2 == (double)-1 && PyErr_Occurred())
    return NULL;

  x3 = (double)_cffi_to_c_double(arg3);
  if (x3 == (double)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, unsigned long);
  if (x4 == (unsigned long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg6, (char **)&x6);
  if (datasize != 0) {
    x6 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg6, (char **)&x6,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg7, (char **)&x7);
  if (datasize != 0) {
    x7 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg7, (char **)&x7,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg8, (char **)&x8);
  if (datasize != 0) {
    x8 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg8, (char **)&x8,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg9, (char **)&x9);
  if (datasize != 0) {
    x9 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg9, (char **)&x9,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg10, (char **)&x10);
  if (datasize != 0) {
    x10 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg10, (char **)&x10,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_nearest_find(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_nearest_find _cffi_d_grib_nearest_find
#endif

static int _cffi_d_grib_nearest_find_multiple(grib_handle const * x0, int x1, double const * x2, double const * x3, long x4, double * x5, double * x6, double * x7, double * x8, int * x9)
{
  return grib_nearest_find_multiple(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_nearest_find_multiple(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  int x1;
  double const * x2;
  double const * x3;
  long x4;
  double * x5;
  double * x6;
  double * x7;
  double * x8;
  int * x9;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;
  PyObject *arg9;

  if (!PyArg_UnpackTuple(args, "grib_nearest_find_multiple", 10, 10, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(135), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(135), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(135), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(135), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = _cffi_to_c_int(arg4, long);
  if (x4 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg6, (char **)&x6);
  if (datasize != 0) {
    x6 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg6, (char **)&x6,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg7, (char **)&x7);
  if (datasize != 0) {
    x7 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg7, (char **)&x7,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(192), arg8, (char **)&x8);
  if (datasize != 0) {
    x8 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(192), arg8, (char **)&x8,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg9, (char **)&x9);
  if (datasize != 0) {
    x9 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg9, (char **)&x9,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_nearest_find_multiple(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_nearest_find_multiple _cffi_d_grib_nearest_find_multiple
#endif

static grib_nearest * _cffi_d_grib_nearest_new(grib_handle const * x0, int * x1)
{
  return grib_nearest_new(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_nearest_new(PyObject *self, PyObject *args)
{
  grib_handle const * x0;
  int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  grib_nearest * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_nearest_new", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_nearest_new(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(337));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_nearest_new _cffi_d_grib_nearest_new
#endif

static char * _cffi_d_grib_samples_path(grib_context const * x0)
{
  return grib_samples_path(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_samples_path(PyObject *self, PyObject *arg0)
{
  grib_context const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_context const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_samples_path(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(47));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_samples_path _cffi_d_grib_samples_path
#endif

static int _cffi_d_grib_set_double(grib_handle * x0, char const * x1, double x2)
{
  return grib_set_double(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_double(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  double x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_set_double", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = (double)_cffi_to_c_double(arg2);
  if (x2 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_double(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_double _cffi_d_grib_set_double
#endif

static int _cffi_d_grib_set_double_array(grib_handle * x0, char const * x1, double const * x2, size_t x3)
{
  return grib_set_double_array(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_double_array(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  double const * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_set_double_array", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(135), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (double const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(135), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_double_array(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_double_array _cffi_d_grib_set_double_array
#endif

static int _cffi_d_grib_set_long(grib_handle * x0, char const * x1, long x2)
{
  return grib_set_long(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_long(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  long x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_set_long", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_long(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_long _cffi_d_grib_set_long
#endif

static int _cffi_d_grib_set_long_array(grib_handle * x0, char const * x1, long const * x2, size_t x3)
{
  return grib_set_long_array(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_long_array(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  long const * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_set_long_array", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(151), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (long const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(151), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_long_array(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_long_array _cffi_d_grib_set_long_array
#endif

static int _cffi_d_grib_set_missing(grib_handle * x0, char const * x1)
{
  return grib_set_missing(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_missing(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "grib_set_missing", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_missing(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_missing _cffi_d_grib_set_missing
#endif

static int _cffi_d_grib_set_string(grib_handle * x0, char const * x1, char const * x2, size_t * x3)
{
  return grib_set_string(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_string(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  char const * x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_set_string", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(130), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(130), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_string(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_string _cffi_d_grib_set_string
#endif

static int _cffi_d_grib_set_string_array(grib_handle * x0, char const * x1, char const * * x2, size_t x3)
{
  return grib_set_string_array(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_string_array(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  char const * x1;
  char const * * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "grib_set_string_array", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(123), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(123), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_string_array(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_string_array _cffi_d_grib_set_string_array
#endif

static int _cffi_d_grib_set_values(grib_handle * x0, grib_values * x1, size_t x2)
{
  return grib_set_values(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_grib_set_values(PyObject *self, PyObject *args)
{
  grib_handle * x0;
  grib_values * x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "grib_set_values", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (grib_handle *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(81), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (grib_values *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(81), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = grib_set_values(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_grib_set_values _cffi_d_grib_set_values
#endif

static int _cffi_d_parse_keyval_string(char const * x0, char * x1, int x2, int x3, grib_values * x4, int * x5)
{
  return parse_keyval_string(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_parse_keyval_string(PyObject *self, PyObject *args)
{
  char const * x0;
  char * x1;
  int x2;
  int x3;
  grib_values * x4;
  int * x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "parse_keyval_string", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(47), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(47), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(81), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (grib_values *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(81), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = parse_keyval_string(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_parse_keyval_string _cffi_d_parse_keyval_string
#endif

static int _cffi_const_GRIB_7777_NOT_FOUND(unsigned long long *o)
{
  int n = (GRIB_7777_NOT_FOUND) <= 0;
  *o = (unsigned long long)((GRIB_7777_NOT_FOUND) | 0);  /* check that GRIB_7777_NOT_FOUND is an integer */
  if (!_cffi_check_int(*o, n, -5))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_ARRAY_TOO_SMALL(unsigned long long *o)
{
  int n = (GRIB_ARRAY_TOO_SMALL) <= 0;
  *o = (unsigned long long)((GRIB_ARRAY_TOO_SMALL) | 0);  /* check that GRIB_ARRAY_TOO_SMALL is an integer */
  if (!_cffi_check_int(*o, n, -6))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_ATTRIBUTE_CLASH(unsigned long long *o)
{
  int n = (GRIB_ATTRIBUTE_CLASH) <= 0;
  *o = (unsigned long long)((GRIB_ATTRIBUTE_CLASH) | 0);  /* check that GRIB_ATTRIBUTE_CLASH is an integer */
  if (!_cffi_check_int(*o, n, -61))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_ATTRIBUTE_NOT_FOUND(unsigned long long *o)
{
  int n = (GRIB_ATTRIBUTE_NOT_FOUND) <= 0;
  *o = (unsigned long long)((GRIB_ATTRIBUTE_NOT_FOUND) | 0);  /* check that GRIB_ATTRIBUTE_NOT_FOUND is an integer */
  if (!_cffi_check_int(*o, n, -63))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_BUFFER_TOO_SMALL(unsigned long long *o)
{
  int n = (GRIB_BUFFER_TOO_SMALL) <= 0;
  *o = (unsigned long long)((GRIB_BUFFER_TOO_SMALL) | 0);  /* check that GRIB_BUFFER_TOO_SMALL is an integer */
  if (!_cffi_check_int(*o, n, -3))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_CODE_NOT_FOUND_IN_TABLE(unsigned long long *o)
{
  int n = (GRIB_CODE_NOT_FOUND_IN_TABLE) <= 0;
  *o = (unsigned long long)((GRIB_CODE_NOT_FOUND_IN_TABLE) | 0);  /* check that GRIB_CODE_NOT_FOUND_IN_TABLE is an integer */
  if (!_cffi_check_int(*o, n, -8))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_CONCEPT_NO_MATCH(unsigned long long *o)
{
  int n = (GRIB_CONCEPT_NO_MATCH) <= 0;
  *o = (unsigned long long)((GRIB_CONCEPT_NO_MATCH) | 0);  /* check that GRIB_CONCEPT_NO_MATCH is an integer */
  if (!_cffi_check_int(*o, n, -36))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_CONSTANT_FIELD(unsigned long long *o)
{
  int n = (GRIB_CONSTANT_FIELD) <= 0;
  *o = (unsigned long long)((GRIB_CONSTANT_FIELD) | 0);  /* check that GRIB_CONSTANT_FIELD is an integer */
  if (!_cffi_check_int(*o, n, -48))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_CORRUPTED_INDEX(unsigned long long *o)
{
  int n = (GRIB_CORRUPTED_INDEX) <= 0;
  *o = (unsigned long long)((GRIB_CORRUPTED_INDEX) | 0);  /* check that GRIB_CORRUPTED_INDEX is an integer */
  if (!_cffi_check_int(*o, n, -52))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_DECODING_ERROR(unsigned long long *o)
{
  int n = (GRIB_DECODING_ERROR) <= 0;
  *o = (unsigned long long)((GRIB_DECODING_ERROR) | 0);  /* check that GRIB_DECODING_ERROR is an integer */
  if (!_cffi_check_int(*o, n, -13))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_DIFFERENT_EDITION(unsigned long long *o)
{
  int n = (GRIB_DIFFERENT_EDITION) <= 0;
  *o = (unsigned long long)((GRIB_DIFFERENT_EDITION) | 0);  /* check that GRIB_DIFFERENT_EDITION is an integer */
  if (!_cffi_check_int(*o, n, -54))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_ENCODING_ERROR(unsigned long long *o)
{
  int n = (GRIB_ENCODING_ERROR) <= 0;
  *o = (unsigned long long)((GRIB_ENCODING_ERROR) | 0);  /* check that GRIB_ENCODING_ERROR is an integer */
  if (!_cffi_check_int(*o, n, -14))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_END(unsigned long long *o)
{
  int n = (GRIB_END) <= 0;
  *o = (unsigned long long)((GRIB_END) | 0);  /* check that GRIB_END is an integer */
  if (!_cffi_check_int(*o, n, -40))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_END_OF_FILE(unsigned long long *o)
{
  int n = (GRIB_END_OF_FILE) <= 0;
  *o = (unsigned long long)((GRIB_END_OF_FILE) | 0);  /* check that GRIB_END_OF_FILE is an integer */
  if (!_cffi_check_int(*o, n, -1))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_END_OF_INDEX(unsigned long long *o)
{
  int n = (GRIB_END_OF_INDEX) <= 0;
  *o = (unsigned long long)((GRIB_END_OF_INDEX) | 0);  /* check that GRIB_END_OF_INDEX is an integer */
  if (!_cffi_check_int(*o, n, -43))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_FILE_NOT_FOUND(unsigned long long *o)
{
  int n = (GRIB_FILE_NOT_FOUND) <= 0;
  *o = (unsigned long long)((GRIB_FILE_NOT_FOUND) | 0);  /* check that GRIB_FILE_NOT_FOUND is an integer */
  if (!_cffi_check_int(*o, n, -7))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_FUNCTIONALITY_NOT_ENABLED(unsigned long long *o)
{
  int n = (GRIB_FUNCTIONALITY_NOT_ENABLED) <= 0;
  *o = (unsigned long long)((GRIB_FUNCTIONALITY_NOT_ENABLED) | 0);  /* check that GRIB_FUNCTIONALITY_NOT_ENABLED is an integer */
  if (!_cffi_check_int(*o, n, -67))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_GEOCALCULUS_PROBLEM(unsigned long long *o)
{
  int n = (GRIB_GEOCALCULUS_PROBLEM) <= 0;
  *o = (unsigned long long)((GRIB_GEOCALCULUS_PROBLEM) | 0);  /* check that GRIB_GEOCALCULUS_PROBLEM is an integer */
  if (!_cffi_check_int(*o, n, -16))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_HASH_ARRAY_NO_MATCH(unsigned long long *o)
{
  int n = (GRIB_HASH_ARRAY_NO_MATCH) <= 0;
  *o = (unsigned long long)((GRIB_HASH_ARRAY_NO_MATCH) | 0);  /* check that GRIB_HASH_ARRAY_NO_MATCH is an integer */
  if (!_cffi_check_int(*o, n, -37))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INTERNAL_ARRAY_TOO_SMALL(unsigned long long *o)
{
  int n = (GRIB_INTERNAL_ARRAY_TOO_SMALL) <= 0;
  *o = (unsigned long long)((GRIB_INTERNAL_ARRAY_TOO_SMALL) | 0);  /* check that GRIB_INTERNAL_ARRAY_TOO_SMALL is an integer */
  if (!_cffi_check_int(*o, n, -46))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INTERNAL_ERROR(unsigned long long *o)
{
  int n = (GRIB_INTERNAL_ERROR) <= 0;
  *o = (unsigned long long)((GRIB_INTERNAL_ERROR) | 0);  /* check that GRIB_INTERNAL_ERROR is an integer */
  if (!_cffi_check_int(*o, n, -2))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_ARGUMENT(unsigned long long *o)
{
  int n = (GRIB_INVALID_ARGUMENT) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_ARGUMENT) | 0);  /* check that GRIB_INVALID_ARGUMENT is an integer */
  if (!_cffi_check_int(*o, n, -19))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_BPV(unsigned long long *o)
{
  int n = (GRIB_INVALID_BPV) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_BPV) | 0);  /* check that GRIB_INVALID_BPV is an integer */
  if (!_cffi_check_int(*o, n, -53))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_FILE(unsigned long long *o)
{
  int n = (GRIB_INVALID_FILE) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_FILE) | 0);  /* check that GRIB_INVALID_FILE is an integer */
  if (!_cffi_check_int(*o, n, -27))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_GRIB(unsigned long long *o)
{
  int n = (GRIB_INVALID_GRIB) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_GRIB) | 0);  /* check that GRIB_INVALID_GRIB is an integer */
  if (!_cffi_check_int(*o, n, -28))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_INDEX(unsigned long long *o)
{
  int n = (GRIB_INVALID_INDEX) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_INDEX) | 0);  /* check that GRIB_INVALID_INDEX is an integer */
  if (!_cffi_check_int(*o, n, -29))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_ITERATOR(unsigned long long *o)
{
  int n = (GRIB_INVALID_ITERATOR) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_ITERATOR) | 0);  /* check that GRIB_INVALID_ITERATOR is an integer */
  if (!_cffi_check_int(*o, n, -30))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_KEYS_ITERATOR(unsigned long long *o)
{
  int n = (GRIB_INVALID_KEYS_ITERATOR) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_KEYS_ITERATOR) | 0);  /* check that GRIB_INVALID_KEYS_ITERATOR is an integer */
  if (!_cffi_check_int(*o, n, -31))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_KEY_VALUE(unsigned long long *o)
{
  int n = (GRIB_INVALID_KEY_VALUE) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_KEY_VALUE) | 0);  /* check that GRIB_INVALID_KEY_VALUE is an integer */
  if (!_cffi_check_int(*o, n, -56))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_MESSAGE(unsigned long long *o)
{
  int n = (GRIB_INVALID_MESSAGE) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_MESSAGE) | 0);  /* check that GRIB_INVALID_MESSAGE is an integer */
  if (!_cffi_check_int(*o, n, -12))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_NEAREST(unsigned long long *o)
{
  int n = (GRIB_INVALID_NEAREST) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_NEAREST) | 0);  /* check that GRIB_INVALID_NEAREST is an integer */
  if (!_cffi_check_int(*o, n, -32))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_ORDERBY(unsigned long long *o)
{
  int n = (GRIB_INVALID_ORDERBY) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_ORDERBY) | 0);  /* check that GRIB_INVALID_ORDERBY is an integer */
  if (!_cffi_check_int(*o, n, -33))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_SECTION_NUMBER(unsigned long long *o)
{
  int n = (GRIB_INVALID_SECTION_NUMBER) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_SECTION_NUMBER) | 0);  /* check that GRIB_INVALID_SECTION_NUMBER is an integer */
  if (!_cffi_check_int(*o, n, -21))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_INVALID_TYPE(unsigned long long *o)
{
  int n = (GRIB_INVALID_TYPE) <= 0;
  *o = (unsigned long long)((GRIB_INVALID_TYPE) | 0);  /* check that GRIB_INVALID_TYPE is an integer */
  if (!_cffi_check_int(*o, n, -24))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_IO_PROBLEM(unsigned long long *o)
{
  int n = (GRIB_IO_PROBLEM) <= 0;
  *o = (unsigned long long)((GRIB_IO_PROBLEM) | 0);  /* check that GRIB_IO_PROBLEM is an integer */
  if (!_cffi_check_int(*o, n, -11))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_KEYS_ITERATOR_SKIP_CODED(unsigned long long *o)
{
  int n = (GRIB_KEYS_ITERATOR_SKIP_CODED) <= 0;
  *o = (unsigned long long)((GRIB_KEYS_ITERATOR_SKIP_CODED) | 0);  /* check that GRIB_KEYS_ITERATOR_SKIP_CODED is an integer */
  if (!_cffi_check_int(*o, n, 8U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_KEYS_ITERATOR_SKIP_COMPUTED(unsigned long long *o)
{
  int n = (GRIB_KEYS_ITERATOR_SKIP_COMPUTED) <= 0;
  *o = (unsigned long long)((GRIB_KEYS_ITERATOR_SKIP_COMPUTED) | 0);  /* check that GRIB_KEYS_ITERATOR_SKIP_COMPUTED is an integer */
  if (!_cffi_check_int(*o, n, 16U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_KEYS_ITERATOR_SKIP_DUPLICATES(unsigned long long *o)
{
  int n = (GRIB_KEYS_ITERATOR_SKIP_DUPLICATES) <= 0;
  *o = (unsigned long long)((GRIB_KEYS_ITERATOR_SKIP_DUPLICATES) | 0);  /* check that GRIB_KEYS_ITERATOR_SKIP_DUPLICATES is an integer */
  if (!_cffi_check_int(*o, n, 32U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_KEYS_ITERATOR_SKIP_EDITION_SPECIFIC(unsigned long long *o)
{
  int n = (GRIB_KEYS_ITERATOR_SKIP_EDITION_SPECIFIC) <= 0;
  *o = (unsigned long long)((GRIB_KEYS_ITERATOR_SKIP_EDITION_SPECIFIC) | 0);  /* check that GRIB_KEYS_ITERATOR_SKIP_EDITION_SPECIFIC is an integer */
  if (!_cffi_check_int(*o, n, 4U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_KEYS_ITERATOR_SKIP_FUNCTION(unsigned long long *o)
{
  int n = (GRIB_KEYS_ITERATOR_SKIP_FUNCTION) <= 0;
  *o = (unsigned long long)((GRIB_KEYS_ITERATOR_SKIP_FUNCTION) | 0);  /* check that GRIB_KEYS_ITERATOR_SKIP_FUNCTION is an integer */
  if (!_cffi_check_int(*o, n, 64U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_KEYS_ITERATOR_SKIP_READ_ONLY(unsigned long long *o)
{
  int n = (GRIB_KEYS_ITERATOR_SKIP_READ_ONLY) <= 0;
  *o = (unsigned long long)((GRIB_KEYS_ITERATOR_SKIP_READ_ONLY) | 0);  /* check that GRIB_KEYS_ITERATOR_SKIP_READ_ONLY is an integer */
  if (!_cffi_check_int(*o, n, 1U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_MESSAGE_MALFORMED(unsigned long long *o)
{
  int n = (GRIB_MESSAGE_MALFORMED) <= 0;
  *o = (unsigned long long)((GRIB_MESSAGE_MALFORMED) | 0);  /* check that GRIB_MESSAGE_MALFORMED is an integer */
  if (!_cffi_check_int(*o, n, -51))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_MESSAGE_TOO_LARGE(unsigned long long *o)
{
  int n = (GRIB_MESSAGE_TOO_LARGE) <= 0;
  *o = (unsigned long long)((GRIB_MESSAGE_TOO_LARGE) | 0);  /* check that GRIB_MESSAGE_TOO_LARGE is an integer */
  if (!_cffi_check_int(*o, n, -47))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_MISSING_BUFR_ENTRY(unsigned long long *o)
{
  int n = (GRIB_MISSING_BUFR_ENTRY) <= 0;
  *o = (unsigned long long)((GRIB_MISSING_BUFR_ENTRY) | 0);  /* check that GRIB_MISSING_BUFR_ENTRY is an integer */
  if (!_cffi_check_int(*o, n, -59))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_MISSING_KEY(unsigned long long *o)
{
  int n = (GRIB_MISSING_KEY) <= 0;
  *o = (unsigned long long)((GRIB_MISSING_KEY) | 0);  /* check that GRIB_MISSING_KEY is an integer */
  if (!_cffi_check_int(*o, n, -34))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NOT_FOUND(unsigned long long *o)
{
  int n = (GRIB_NOT_FOUND) <= 0;
  *o = (unsigned long long)((GRIB_NOT_FOUND) | 0);  /* check that GRIB_NOT_FOUND is an integer */
  if (!_cffi_check_int(*o, n, -10))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NOT_IMPLEMENTED(unsigned long long *o)
{
  int n = (GRIB_NOT_IMPLEMENTED) <= 0;
  *o = (unsigned long long)((GRIB_NOT_IMPLEMENTED) | 0);  /* check that GRIB_NOT_IMPLEMENTED is an integer */
  if (!_cffi_check_int(*o, n, -4))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NO_DEFINITIONS(unsigned long long *o)
{
  int n = (GRIB_NO_DEFINITIONS) <= 0;
  *o = (unsigned long long)((GRIB_NO_DEFINITIONS) | 0);  /* check that GRIB_NO_DEFINITIONS is an integer */
  if (!_cffi_check_int(*o, n, -38))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NO_MORE_IN_SET(unsigned long long *o)
{
  int n = (GRIB_NO_MORE_IN_SET) <= 0;
  *o = (unsigned long long)((GRIB_NO_MORE_IN_SET) | 0);  /* check that GRIB_NO_MORE_IN_SET is an integer */
  if (!_cffi_check_int(*o, n, -15))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NO_VALUES(unsigned long long *o)
{
  int n = (GRIB_NO_VALUES) <= 0;
  *o = (unsigned long long)((GRIB_NO_VALUES) | 0);  /* check that GRIB_NO_VALUES is an integer */
  if (!_cffi_check_int(*o, n, -41))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NULL_HANDLE(unsigned long long *o)
{
  int n = (GRIB_NULL_HANDLE) <= 0;
  *o = (unsigned long long)((GRIB_NULL_HANDLE) | 0);  /* check that GRIB_NULL_HANDLE is an integer */
  if (!_cffi_check_int(*o, n, -20))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NULL_INDEX(unsigned long long *o)
{
  int n = (GRIB_NULL_INDEX) <= 0;
  *o = (unsigned long long)((GRIB_NULL_INDEX) | 0);  /* check that GRIB_NULL_INDEX is an integer */
  if (!_cffi_check_int(*o, n, -44))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_NULL_POINTER(unsigned long long *o)
{
  int n = (GRIB_NULL_POINTER) <= 0;
  *o = (unsigned long long)((GRIB_NULL_POINTER) | 0);  /* check that GRIB_NULL_POINTER is an integer */
  if (!_cffi_check_int(*o, n, -60))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_OUT_OF_AREA(unsigned long long *o)
{
  int n = (GRIB_OUT_OF_AREA) <= 0;
  *o = (unsigned long long)((GRIB_OUT_OF_AREA) | 0);  /* check that GRIB_OUT_OF_AREA is an integer */
  if (!_cffi_check_int(*o, n, -35))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_OUT_OF_MEMORY(unsigned long long *o)
{
  int n = (GRIB_OUT_OF_MEMORY) <= 0;
  *o = (unsigned long long)((GRIB_OUT_OF_MEMORY) | 0);  /* check that GRIB_OUT_OF_MEMORY is an integer */
  if (!_cffi_check_int(*o, n, -17))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_OUT_OF_RANGE(unsigned long long *o)
{
  int n = (GRIB_OUT_OF_RANGE) <= 0;
  *o = (unsigned long long)((GRIB_OUT_OF_RANGE) | 0);  /* check that GRIB_OUT_OF_RANGE is an integer */
  if (!_cffi_check_int(*o, n, -65))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_PREMATURE_END_OF_FILE(unsigned long long *o)
{
  int n = (GRIB_PREMATURE_END_OF_FILE) <= 0;
  *o = (unsigned long long)((GRIB_PREMATURE_END_OF_FILE) | 0);  /* check that GRIB_PREMATURE_END_OF_FILE is an integer */
  if (!_cffi_check_int(*o, n, -45))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_READ_ONLY(unsigned long long *o)
{
  int n = (GRIB_READ_ONLY) <= 0;
  *o = (unsigned long long)((GRIB_READ_ONLY) | 0);  /* check that GRIB_READ_ONLY is an integer */
  if (!_cffi_check_int(*o, n, -18))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_STRING_TOO_SMALL(unsigned long long *o)
{
  int n = (GRIB_STRING_TOO_SMALL) <= 0;
  *o = (unsigned long long)((GRIB_STRING_TOO_SMALL) | 0);  /* check that GRIB_STRING_TOO_SMALL is an integer */
  if (!_cffi_check_int(*o, n, -57))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_SUCCESS(unsigned long long *o)
{
  int n = (GRIB_SUCCESS) <= 0;
  *o = (unsigned long long)((GRIB_SUCCESS) | 0);  /* check that GRIB_SUCCESS is an integer */
  if (!_cffi_check_int(*o, n, 0))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_SWITCH_NO_MATCH(unsigned long long *o)
{
  int n = (GRIB_SWITCH_NO_MATCH) <= 0;
  *o = (unsigned long long)((GRIB_SWITCH_NO_MATCH) | 0);  /* check that GRIB_SWITCH_NO_MATCH is an integer */
  if (!_cffi_check_int(*o, n, -49))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TOO_MANY_ATTRIBUTES(unsigned long long *o)
{
  int n = (GRIB_TOO_MANY_ATTRIBUTES) <= 0;
  *o = (unsigned long long)((GRIB_TOO_MANY_ATTRIBUTES) | 0);  /* check that GRIB_TOO_MANY_ATTRIBUTES is an integer */
  if (!_cffi_check_int(*o, n, -62))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_BYTES(unsigned long long *o)
{
  int n = (GRIB_TYPE_BYTES) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_BYTES) | 0);  /* check that GRIB_TYPE_BYTES is an integer */
  if (!_cffi_check_int(*o, n, 4U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_DOUBLE(unsigned long long *o)
{
  int n = (GRIB_TYPE_DOUBLE) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_DOUBLE) | 0);  /* check that GRIB_TYPE_DOUBLE is an integer */
  if (!_cffi_check_int(*o, n, 2U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_LABEL(unsigned long long *o)
{
  int n = (GRIB_TYPE_LABEL) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_LABEL) | 0);  /* check that GRIB_TYPE_LABEL is an integer */
  if (!_cffi_check_int(*o, n, 6U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_LONG(unsigned long long *o)
{
  int n = (GRIB_TYPE_LONG) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_LONG) | 0);  /* check that GRIB_TYPE_LONG is an integer */
  if (!_cffi_check_int(*o, n, 1U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_MISSING(unsigned long long *o)
{
  int n = (GRIB_TYPE_MISSING) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_MISSING) | 0);  /* check that GRIB_TYPE_MISSING is an integer */
  if (!_cffi_check_int(*o, n, 7U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_SECTION(unsigned long long *o)
{
  int n = (GRIB_TYPE_SECTION) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_SECTION) | 0);  /* check that GRIB_TYPE_SECTION is an integer */
  if (!_cffi_check_int(*o, n, 5U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_STRING(unsigned long long *o)
{
  int n = (GRIB_TYPE_STRING) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_STRING) | 0);  /* check that GRIB_TYPE_STRING is an integer */
  if (!_cffi_check_int(*o, n, 3U))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_TYPE_UNDEFINED(unsigned long long *o)
{
  int n = (GRIB_TYPE_UNDEFINED) <= 0;
  *o = (unsigned long long)((GRIB_TYPE_UNDEFINED) | 0);  /* check that GRIB_TYPE_UNDEFINED is an integer */
  if (!_cffi_check_int(*o, n, 0))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_UNDERFLOW(unsigned long long *o)
{
  int n = (GRIB_UNDERFLOW) <= 0;
  *o = (unsigned long long)((GRIB_UNDERFLOW) | 0);  /* check that GRIB_UNDERFLOW is an integer */
  if (!_cffi_check_int(*o, n, -50))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_UNSUPPORTED_EDITION(unsigned long long *o)
{
  int n = (GRIB_UNSUPPORTED_EDITION) <= 0;
  *o = (unsigned long long)((GRIB_UNSUPPORTED_EDITION) | 0);  /* check that GRIB_UNSUPPORTED_EDITION is an integer */
  if (!_cffi_check_int(*o, n, -64))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_VALUE_CANNOT_BE_MISSING(unsigned long long *o)
{
  int n = (GRIB_VALUE_CANNOT_BE_MISSING) <= 0;
  *o = (unsigned long long)((GRIB_VALUE_CANNOT_BE_MISSING) | 0);  /* check that GRIB_VALUE_CANNOT_BE_MISSING is an integer */
  if (!_cffi_check_int(*o, n, -22))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_VALUE_DIFFERENT(unsigned long long *o)
{
  int n = (GRIB_VALUE_DIFFERENT) <= 0;
  *o = (unsigned long long)((GRIB_VALUE_DIFFERENT) | 0);  /* check that GRIB_VALUE_DIFFERENT is an integer */
  if (!_cffi_check_int(*o, n, -55))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_ARRAY_SIZE(unsigned long long *o)
{
  int n = (GRIB_WRONG_ARRAY_SIZE) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_ARRAY_SIZE) | 0);  /* check that GRIB_WRONG_ARRAY_SIZE is an integer */
  if (!_cffi_check_int(*o, n, -9))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_BITMAP_SIZE(unsigned long long *o)
{
  int n = (GRIB_WRONG_BITMAP_SIZE) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_BITMAP_SIZE) | 0);  /* check that GRIB_WRONG_BITMAP_SIZE is an integer */
  if (!_cffi_check_int(*o, n, -66))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_CONVERSION(unsigned long long *o)
{
  int n = (GRIB_WRONG_CONVERSION) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_CONVERSION) | 0);  /* check that GRIB_WRONG_CONVERSION is an integer */
  if (!_cffi_check_int(*o, n, -58))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_GRID(unsigned long long *o)
{
  int n = (GRIB_WRONG_GRID) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_GRID) | 0);  /* check that GRIB_WRONG_GRID is an integer */
  if (!_cffi_check_int(*o, n, -42))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_LENGTH(unsigned long long *o)
{
  int n = (GRIB_WRONG_LENGTH) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_LENGTH) | 0);  /* check that GRIB_WRONG_LENGTH is an integer */
  if (!_cffi_check_int(*o, n, -23))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_STEP(unsigned long long *o)
{
  int n = (GRIB_WRONG_STEP) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_STEP) | 0);  /* check that GRIB_WRONG_STEP is an integer */
  if (!_cffi_check_int(*o, n, -25))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_STEP_UNIT(unsigned long long *o)
{
  int n = (GRIB_WRONG_STEP_UNIT) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_STEP_UNIT) | 0);  /* check that GRIB_WRONG_STEP_UNIT is an integer */
  if (!_cffi_check_int(*o, n, -26))
    n |= 2;
  return n;
}

static int _cffi_const_GRIB_WRONG_TYPE(unsigned long long *o)
{
  int n = (GRIB_WRONG_TYPE) <= 0;
  *o = (unsigned long long)((GRIB_WRONG_TYPE) | 0);  /* check that GRIB_WRONG_TYPE is an integer */
  if (!_cffi_check_int(*o, n, -39))
    n |= 2;
  return n;
}

_CFFI_UNUSED_FN
static void _cffi_checkfld__codes_bufr_header(codes_bufr_header *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->message_offset) | 0);  /* check that 'codes_bufr_header.message_offset' is an integer */
  (void)((p->message_size) | 0);  /* check that 'codes_bufr_header.message_size' is an integer */
  (void)((p->edition) | 0);  /* check that 'codes_bufr_header.edition' is an integer */
  (void)((p->masterTableNumber) | 0);  /* check that 'codes_bufr_header.masterTableNumber' is an integer */
  (void)((p->bufrHeaderSubCentre) | 0);  /* check that 'codes_bufr_header.bufrHeaderSubCentre' is an integer */
  (void)((p->bufrHeaderCentre) | 0);  /* check that 'codes_bufr_header.bufrHeaderCentre' is an integer */
  (void)((p->updateSequenceNumber) | 0);  /* check that 'codes_bufr_header.updateSequenceNumber' is an integer */
  (void)((p->dataCategory) | 0);  /* check that 'codes_bufr_header.dataCategory' is an integer */
  (void)((p->dataSubCategory) | 0);  /* check that 'codes_bufr_header.dataSubCategory' is an integer */
  (void)((p->masterTablesVersionNumber) | 0);  /* check that 'codes_bufr_header.masterTablesVersionNumber' is an integer */
  (void)((p->localTablesVersionNumber) | 0);  /* check that 'codes_bufr_header.localTablesVersionNumber' is an integer */
  (void)((p->typicalYear) | 0);  /* check that 'codes_bufr_header.typicalYear' is an integer */
  (void)((p->typicalMonth) | 0);  /* check that 'codes_bufr_header.typicalMonth' is an integer */
  (void)((p->typicalDay) | 0);  /* check that 'codes_bufr_header.typicalDay' is an integer */
  (void)((p->typicalHour) | 0);  /* check that 'codes_bufr_header.typicalHour' is an integer */
  (void)((p->typicalMinute) | 0);  /* check that 'codes_bufr_header.typicalMinute' is an integer */
  (void)((p->typicalSecond) | 0);  /* check that 'codes_bufr_header.typicalSecond' is an integer */
  (void)((p->typicalDate) | 0);  /* check that 'codes_bufr_header.typicalDate' is an integer */
  (void)((p->typicalTime) | 0);  /* check that 'codes_bufr_header.typicalTime' is an integer */
  (void)((p->internationalDataSubCategory) | 0);  /* check that 'codes_bufr_header.internationalDataSubCategory' is an integer */
  (void)((p->localSectionPresent) | 0);  /* check that 'codes_bufr_header.localSectionPresent' is an integer */
  (void)((p->ecmwfLocalSectionPresent) | 0);  /* check that 'codes_bufr_header.ecmwfLocalSectionPresent' is an integer */
  (void)((p->rdbType) | 0);  /* check that 'codes_bufr_header.rdbType' is an integer */
  (void)((p->oldSubtype) | 0);  /* check that 'codes_bufr_header.oldSubtype' is an integer */
  (void)((p->rdbSubtype) | 0);  /* check that 'codes_bufr_header.rdbSubtype' is an integer */
  { char(*tmp)[9] = &p->ident; (void)tmp; }
  (void)((p->localYear) | 0);  /* check that 'codes_bufr_header.localYear' is an integer */
  (void)((p->localMonth) | 0);  /* check that 'codes_bufr_header.localMonth' is an integer */
  (void)((p->localDay) | 0);  /* check that 'codes_bufr_header.localDay' is an integer */
  (void)((p->localHour) | 0);  /* check that 'codes_bufr_header.localHour' is an integer */
  (void)((p->localMinute) | 0);  /* check that 'codes_bufr_header.localMinute' is an integer */
  (void)((p->localSecond) | 0);  /* check that 'codes_bufr_header.localSecond' is an integer */
  (void)((p->rdbtimeDay) | 0);  /* check that 'codes_bufr_header.rdbtimeDay' is an integer */
  (void)((p->rdbtimeHour) | 0);  /* check that 'codes_bufr_header.rdbtimeHour' is an integer */
  (void)((p->rdbtimeMinute) | 0);  /* check that 'codes_bufr_header.rdbtimeMinute' is an integer */
  (void)((p->rdbtimeSecond) | 0);  /* check that 'codes_bufr_header.rdbtimeSecond' is an integer */
  (void)((p->rectimeDay) | 0);  /* check that 'codes_bufr_header.rectimeDay' is an integer */
  (void)((p->rectimeHour) | 0);  /* check that 'codes_bufr_header.rectimeHour' is an integer */
  (void)((p->rectimeMinute) | 0);  /* check that 'codes_bufr_header.rectimeMinute' is an integer */
  (void)((p->rectimeSecond) | 0);  /* check that 'codes_bufr_header.rectimeSecond' is an integer */
  (void)((p->restricted) | 0);  /* check that 'codes_bufr_header.restricted' is an integer */
  (void)((p->isSatellite) | 0);  /* check that 'codes_bufr_header.isSatellite' is an integer */
  { double *tmp = &p->localLongitude1; (void)tmp; }
  { double *tmp = &p->localLatitude1; (void)tmp; }
  { double *tmp = &p->localLongitude2; (void)tmp; }
  { double *tmp = &p->localLatitude2; (void)tmp; }
  { double *tmp = &p->localLatitude; (void)tmp; }
  { double *tmp = &p->localLongitude; (void)tmp; }
  (void)((p->localNumberOfObservations) | 0);  /* check that 'codes_bufr_header.localNumberOfObservations' is an integer */
  (void)((p->satelliteID) | 0);  /* check that 'codes_bufr_header.satelliteID' is an integer */
  (void)((p->qualityControl) | 0);  /* check that 'codes_bufr_header.qualityControl' is an integer */
  (void)((p->newSubtype) | 0);  /* check that 'codes_bufr_header.newSubtype' is an integer */
  (void)((p->daLoop) | 0);  /* check that 'codes_bufr_header.daLoop' is an integer */
  (void)((p->numberOfSubsets) | 0);  /* check that 'codes_bufr_header.numberOfSubsets' is an integer */
  (void)((p->observedData) | 0);  /* check that 'codes_bufr_header.observedData' is an integer */
  (void)((p->compressedData) | 0);  /* check that 'codes_bufr_header.compressedData' is an integer */
}
struct _cffi_align__codes_bufr_header { char x; codes_bufr_header y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__grib_values(grib_values *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char const * *tmp = &p->name; (void)tmp; }
  (void)((p->type) | 0);  /* check that 'grib_values.type' is an integer */
  (void)((p->long_value) | 0);  /* check that 'grib_values.long_value' is an integer */
  { double *tmp = &p->double_value; (void)tmp; }
  { char const * *tmp = &p->string_value; (void)tmp; }
  (void)((p->error) | 0);  /* check that 'grib_values.error' is an integer */
  (void)((p->has_value) | 0);  /* check that 'grib_values.has_value' is an integer */
  (void)((p->equal) | 0);  /* check that 'grib_values.equal' is an integer */
  { grib_values * *tmp = &p->next; (void)tmp; }
}
struct _cffi_align__grib_values { char x; grib_values y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "GRIB_7777_NOT_FOUND", (void *)_cffi_const_GRIB_7777_NOT_FOUND, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_ARRAY_TOO_SMALL", (void *)_cffi_const_GRIB_ARRAY_TOO_SMALL, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_ATTRIBUTE_CLASH", (void *)_cffi_const_GRIB_ATTRIBUTE_CLASH, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_ATTRIBUTE_NOT_FOUND", (void *)_cffi_const_GRIB_ATTRIBUTE_NOT_FOUND, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_BUFFER_TOO_SMALL", (void *)_cffi_const_GRIB_BUFFER_TOO_SMALL, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_CODE_NOT_FOUND_IN_TABLE", (void *)_cffi_const_GRIB_CODE_NOT_FOUND_IN_TABLE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_CONCEPT_NO_MATCH", (void *)_cffi_const_GRIB_CONCEPT_NO_MATCH, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_CONSTANT_FIELD", (void *)_cffi_const_GRIB_CONSTANT_FIELD, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_CORRUPTED_INDEX", (void *)_cffi_const_GRIB_CORRUPTED_INDEX, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_DECODING_ERROR", (void *)_cffi_const_GRIB_DECODING_ERROR, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_DIFFERENT_EDITION", (void *)_cffi_const_GRIB_DIFFERENT_EDITION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_ENCODING_ERROR", (void *)_cffi_const_GRIB_ENCODING_ERROR, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_END", (void *)_cffi_const_GRIB_END, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_END_OF_FILE", (void *)_cffi_const_GRIB_END_OF_FILE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_END_OF_INDEX", (void *)_cffi_const_GRIB_END_OF_INDEX, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_FILE_NOT_FOUND", (void *)_cffi_const_GRIB_FILE_NOT_FOUND, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_FUNCTIONALITY_NOT_ENABLED", (void *)_cffi_const_GRIB_FUNCTIONALITY_NOT_ENABLED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_GEOCALCULUS_PROBLEM", (void *)_cffi_const_GRIB_GEOCALCULUS_PROBLEM, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_HASH_ARRAY_NO_MATCH", (void *)_cffi_const_GRIB_HASH_ARRAY_NO_MATCH, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INTERNAL_ARRAY_TOO_SMALL", (void *)_cffi_const_GRIB_INTERNAL_ARRAY_TOO_SMALL, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INTERNAL_ERROR", (void *)_cffi_const_GRIB_INTERNAL_ERROR, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_ARGUMENT", (void *)_cffi_const_GRIB_INVALID_ARGUMENT, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_BPV", (void *)_cffi_const_GRIB_INVALID_BPV, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_FILE", (void *)_cffi_const_GRIB_INVALID_FILE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_GRIB", (void *)_cffi_const_GRIB_INVALID_GRIB, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_INDEX", (void *)_cffi_const_GRIB_INVALID_INDEX, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_ITERATOR", (void *)_cffi_const_GRIB_INVALID_ITERATOR, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_KEYS_ITERATOR", (void *)_cffi_const_GRIB_INVALID_KEYS_ITERATOR, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_KEY_VALUE", (void *)_cffi_const_GRIB_INVALID_KEY_VALUE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_MESSAGE", (void *)_cffi_const_GRIB_INVALID_MESSAGE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_NEAREST", (void *)_cffi_const_GRIB_INVALID_NEAREST, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_ORDERBY", (void *)_cffi_const_GRIB_INVALID_ORDERBY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_SECTION_NUMBER", (void *)_cffi_const_GRIB_INVALID_SECTION_NUMBER, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_INVALID_TYPE", (void *)_cffi_const_GRIB_INVALID_TYPE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_IO_PROBLEM", (void *)_cffi_const_GRIB_IO_PROBLEM, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_KEYS_ITERATOR_SKIP_CODED", (void *)_cffi_const_GRIB_KEYS_ITERATOR_SKIP_CODED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_KEYS_ITERATOR_SKIP_COMPUTED", (void *)_cffi_const_GRIB_KEYS_ITERATOR_SKIP_COMPUTED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_KEYS_ITERATOR_SKIP_DUPLICATES", (void *)_cffi_const_GRIB_KEYS_ITERATOR_SKIP_DUPLICATES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_KEYS_ITERATOR_SKIP_EDITION_SPECIFIC", (void *)_cffi_const_GRIB_KEYS_ITERATOR_SKIP_EDITION_SPECIFIC, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_KEYS_ITERATOR_SKIP_FUNCTION", (void *)_cffi_const_GRIB_KEYS_ITERATOR_SKIP_FUNCTION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_KEYS_ITERATOR_SKIP_READ_ONLY", (void *)_cffi_const_GRIB_KEYS_ITERATOR_SKIP_READ_ONLY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_MESSAGE_MALFORMED", (void *)_cffi_const_GRIB_MESSAGE_MALFORMED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_MESSAGE_TOO_LARGE", (void *)_cffi_const_GRIB_MESSAGE_TOO_LARGE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_MISSING_BUFR_ENTRY", (void *)_cffi_const_GRIB_MISSING_BUFR_ENTRY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_MISSING_KEY", (void *)_cffi_const_GRIB_MISSING_KEY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NOT_FOUND", (void *)_cffi_const_GRIB_NOT_FOUND, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NOT_IMPLEMENTED", (void *)_cffi_const_GRIB_NOT_IMPLEMENTED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NO_DEFINITIONS", (void *)_cffi_const_GRIB_NO_DEFINITIONS, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NO_MORE_IN_SET", (void *)_cffi_const_GRIB_NO_MORE_IN_SET, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NO_VALUES", (void *)_cffi_const_GRIB_NO_VALUES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NULL_HANDLE", (void *)_cffi_const_GRIB_NULL_HANDLE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NULL_INDEX", (void *)_cffi_const_GRIB_NULL_INDEX, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_NULL_POINTER", (void *)_cffi_const_GRIB_NULL_POINTER, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_OUT_OF_AREA", (void *)_cffi_const_GRIB_OUT_OF_AREA, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_OUT_OF_MEMORY", (void *)_cffi_const_GRIB_OUT_OF_MEMORY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_OUT_OF_RANGE", (void *)_cffi_const_GRIB_OUT_OF_RANGE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_PREMATURE_END_OF_FILE", (void *)_cffi_const_GRIB_PREMATURE_END_OF_FILE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_READ_ONLY", (void *)_cffi_const_GRIB_READ_ONLY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_STRING_TOO_SMALL", (void *)_cffi_const_GRIB_STRING_TOO_SMALL, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_SUCCESS", (void *)_cffi_const_GRIB_SUCCESS, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_SWITCH_NO_MATCH", (void *)_cffi_const_GRIB_SWITCH_NO_MATCH, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TOO_MANY_ATTRIBUTES", (void *)_cffi_const_GRIB_TOO_MANY_ATTRIBUTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_BYTES", (void *)_cffi_const_GRIB_TYPE_BYTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_DOUBLE", (void *)_cffi_const_GRIB_TYPE_DOUBLE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_LABEL", (void *)_cffi_const_GRIB_TYPE_LABEL, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_LONG", (void *)_cffi_const_GRIB_TYPE_LONG, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_MISSING", (void *)_cffi_const_GRIB_TYPE_MISSING, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_SECTION", (void *)_cffi_const_GRIB_TYPE_SECTION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_STRING", (void *)_cffi_const_GRIB_TYPE_STRING, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_TYPE_UNDEFINED", (void *)_cffi_const_GRIB_TYPE_UNDEFINED, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_UNDERFLOW", (void *)_cffi_const_GRIB_UNDERFLOW, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_UNSUPPORTED_EDITION", (void *)_cffi_const_GRIB_UNSUPPORTED_EDITION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_VALUE_CANNOT_BE_MISSING", (void *)_cffi_const_GRIB_VALUE_CANNOT_BE_MISSING, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_VALUE_DIFFERENT", (void *)_cffi_const_GRIB_VALUE_DIFFERENT, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_ARRAY_SIZE", (void *)_cffi_const_GRIB_WRONG_ARRAY_SIZE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_BITMAP_SIZE", (void *)_cffi_const_GRIB_WRONG_BITMAP_SIZE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_CONVERSION", (void *)_cffi_const_GRIB_WRONG_CONVERSION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_GRID", (void *)_cffi_const_GRIB_WRONG_GRID, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_LENGTH", (void *)_cffi_const_GRIB_WRONG_LENGTH, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_STEP", (void *)_cffi_const_GRIB_WRONG_STEP, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_STEP_UNIT", (void *)_cffi_const_GRIB_WRONG_STEP_UNIT, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "GRIB_WRONG_TYPE", (void *)_cffi_const_GRIB_WRONG_TYPE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "PRODUCT_ANY", (void *)_cffi_const_PRODUCT_ANY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PRODUCT_BUFR", (void *)_cffi_const_PRODUCT_BUFR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PRODUCT_GRIB", (void *)_cffi_const_PRODUCT_GRIB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PRODUCT_GTS", (void *)_cffi_const_PRODUCT_GTS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PRODUCT_METAR", (void *)_cffi_const_PRODUCT_METAR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PRODUCT_TAF", (void *)_cffi_const_PRODUCT_TAF, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "codes_bufr_copy_data", (void *)_cffi_f_codes_bufr_copy_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 159), (void *)_cffi_d_codes_bufr_copy_data },
  { "codes_bufr_extract_headers_malloc", (void *)_cffi_f_codes_bufr_extract_headers_malloc, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 106), (void *)_cffi_d_codes_bufr_extract_headers_malloc },
  { "codes_bufr_handle_new_from_samples", (void *)_cffi_f_codes_bufr_handle_new_from_samples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 29), (void *)_cffi_d_codes_bufr_handle_new_from_samples },
  { "codes_bufr_key_is_coordinate", (void *)_cffi_f_codes_bufr_key_is_coordinate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 206), (void *)_cffi_d_codes_bufr_key_is_coordinate },
  { "codes_bufr_key_is_header", (void *)_cffi_f_codes_bufr_key_is_header, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 206), (void *)_cffi_d_codes_bufr_key_is_header },
  { "codes_bufr_keys_iterator_delete", (void *)_cffi_f_codes_bufr_keys_iterator_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 73), (void *)_cffi_d_codes_bufr_keys_iterator_delete },
  { "codes_bufr_keys_iterator_get_name", (void *)_cffi_f_codes_bufr_keys_iterator_get_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 4), (void *)_cffi_d_codes_bufr_keys_iterator_get_name },
  { "codes_bufr_keys_iterator_new", (void *)_cffi_f_codes_bufr_keys_iterator_new, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_codes_bufr_keys_iterator_new },
  { "codes_bufr_keys_iterator_next", (void *)_cffi_f_codes_bufr_keys_iterator_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 73), (void *)_cffi_d_codes_bufr_keys_iterator_next },
  { "codes_bufr_keys_iterator_rewind", (void *)_cffi_f_codes_bufr_keys_iterator_rewind, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 73), (void *)_cffi_d_codes_bufr_keys_iterator_rewind },
  { "codes_bufr_multi_element_constant_arrays_off", (void *)_cffi_f_codes_bufr_multi_element_constant_arrays_off, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_codes_bufr_multi_element_constant_arrays_off },
  { "codes_bufr_multi_element_constant_arrays_on", (void *)_cffi_f_codes_bufr_multi_element_constant_arrays_on, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_codes_bufr_multi_element_constant_arrays_on },
  { "codes_definition_path", (void *)_cffi_f_codes_definition_path, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_codes_definition_path },
  { "codes_extract_offsets_malloc", (void *)_cffi_f_codes_extract_offsets_malloc, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 89), (void *)_cffi_d_codes_extract_offsets_malloc },
  { "codes_extract_offsets_sizes_malloc", (void *)_cffi_f_codes_extract_offsets_sizes_malloc, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 97), (void *)_cffi_d_codes_extract_offsets_sizes_malloc },
  { "codes_handle_new_from_file", (void *)_cffi_f_codes_handle_new_from_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 18), (void *)_cffi_d_codes_handle_new_from_file },
  { "codes_handle_new_from_samples", (void *)_cffi_f_codes_handle_new_from_samples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 29), (void *)_cffi_d_codes_handle_new_from_samples },
  { "codes_samples_path", (void *)_cffi_f_codes_samples_path, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_codes_samples_path },
  { "grib_context_delete", (void *)_cffi_f_grib_context_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_grib_context_delete },
  { "grib_context_get_default", (void *)_cffi_f_grib_context_get_default, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 16), (void *)_cffi_d_grib_context_get_default },
  { "grib_context_set_definitions_path", (void *)_cffi_f_grib_context_set_definitions_path, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 365), (void *)_cffi_d_grib_context_set_definitions_path },
  { "grib_context_set_samples_path", (void *)_cffi_f_grib_context_set_samples_path, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 365), (void *)_cffi_d_grib_context_set_samples_path },
  { "grib_copy_namespace", (void *)_cffi_f_grib_copy_namespace, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 143), (void *)_cffi_d_grib_copy_namespace },
  { "grib_count_in_file", (void *)_cffi_f_grib_count_in_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 84), (void *)_cffi_d_grib_count_in_file },
  { "grib_definition_path", (void *)_cffi_f_grib_definition_path, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_grib_definition_path },
  { "grib_dump_content", (void *)_cffi_f_grib_dump_content, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 369), (void *)_cffi_d_grib_dump_content },
  { "grib_get_api_version", (void *)_cffi_f_grib_get_api_version, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 356), (void *)_cffi_d_grib_get_api_version },
  { "grib_get_data", (void *)_cffi_f_grib_get_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 240), (void *)_cffi_d_grib_get_data },
  { "grib_get_double", (void *)_cffi_f_grib_get_double, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 189), (void *)_cffi_d_grib_get_double },
  { "grib_get_double_array", (void *)_cffi_f_grib_get_double_array, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 194), (void *)_cffi_d_grib_get_double_array },
  { "grib_get_double_element", (void *)_cffi_f_grib_get_double_element, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 218), (void *)_cffi_d_grib_get_double_element },
  { "grib_get_double_elements", (void *)_cffi_f_grib_get_double_elements, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 211), (void *)_cffi_d_grib_get_double_elements },
  { "grib_get_error_message", (void *)_cffi_f_grib_get_error_message, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 13), (void *)_cffi_d_grib_get_error_message },
  { "grib_get_float_array", (void *)_cffi_f_grib_get_float_array, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 200), (void *)_cffi_d_grib_get_float_array },
  { "grib_get_gaussian_latitudes", (void *)_cffi_f_grib_get_gaussian_latitudes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 352), (void *)_cffi_d_grib_get_gaussian_latitudes },
  { "grib_get_length", (void *)_cffi_f_grib_get_length, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 235), (void *)_cffi_d_grib_get_length },
  { "grib_get_long", (void *)_cffi_f_grib_get_long, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 224), (void *)_cffi_d_grib_get_long },
  { "grib_get_long_array", (void *)_cffi_f_grib_get_long_array, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 229), (void *)_cffi_d_grib_get_long_array },
  { "grib_get_message", (void *)_cffi_f_grib_get_message, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 266), (void *)_cffi_d_grib_get_message },
  { "grib_get_message_offset", (void *)_cffi_f_grib_get_message_offset, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 258), (void *)_cffi_d_grib_get_message_offset },
  { "grib_get_message_size", (void *)_cffi_f_grib_get_message_size, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 262), (void *)_cffi_d_grib_get_message_size },
  { "grib_get_native_type", (void *)_cffi_f_grib_get_native_type, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 206), (void *)_cffi_d_grib_get_native_type },
  { "grib_get_size", (void *)_cffi_f_grib_get_size, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 235), (void *)_cffi_d_grib_get_size },
  { "grib_get_string", (void *)_cffi_f_grib_get_string, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 183), (void *)_cffi_d_grib_get_string },
  { "grib_get_string_array", (void *)_cffi_f_grib_get_string_array, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 177), (void *)_cffi_d_grib_get_string_array },
  { "grib_gribex_mode_off", (void *)_cffi_f_grib_gribex_mode_off, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_grib_gribex_mode_off },
  { "grib_gribex_mode_on", (void *)_cffi_f_grib_gribex_mode_on, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_grib_gribex_mode_on },
  { "grib_gts_header_off", (void *)_cffi_f_grib_gts_header_off, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_grib_gts_header_off },
  { "grib_gts_header_on", (void *)_cffi_f_grib_gts_header_on, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_grib_gts_header_on },
  { "grib_handle_clone", (void *)_cffi_f_grib_handle_clone, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 38), (void *)_cffi_d_grib_handle_clone },
  { "grib_handle_clone_headers_only", (void *)_cffi_f_grib_handle_clone_headers_only, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 38), (void *)_cffi_d_grib_handle_clone_headers_only },
  { "grib_handle_delete", (void *)_cffi_f_grib_handle_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 113), (void *)_cffi_d_grib_handle_delete },
  { "grib_handle_new_from_file", (void *)_cffi_f_grib_handle_new_from_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 24), (void *)_cffi_d_grib_handle_new_from_file },
  { "grib_handle_new_from_index", (void *)_cffi_f_grib_handle_new_from_index, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 41), (void *)_cffi_d_grib_handle_new_from_index },
  { "grib_handle_new_from_message_copy", (void *)_cffi_f_grib_handle_new_from_message_copy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 33), (void *)_cffi_d_grib_handle_new_from_message_copy },
  { "grib_handle_new_from_samples", (void *)_cffi_f_grib_handle_new_from_samples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 29), (void *)_cffi_d_grib_handle_new_from_samples },
  { "grib_index_add_file", (void *)_cffi_f_grib_index_add_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 271), (void *)_cffi_d_grib_index_add_file },
  { "grib_index_delete", (void *)_cffi_f_grib_index_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 376), (void *)_cffi_d_grib_index_delete },
  { "grib_index_get_double", (void *)_cffi_f_grib_index_get_double, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 296), (void *)_cffi_d_grib_index_get_double },
  { "grib_index_get_long", (void *)_cffi_f_grib_index_get_long, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 302), (void *)_cffi_d_grib_index_get_long },
  { "grib_index_get_size", (void *)_cffi_f_grib_index_get_size, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 308), (void *)_cffi_d_grib_index_get_size },
  { "grib_index_get_string", (void *)_cffi_f_grib_index_get_string, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 290), (void *)_cffi_d_grib_index_get_string },
  { "grib_index_new_from_file", (void *)_cffi_f_grib_index_new_from_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 45), (void *)_cffi_d_grib_index_new_from_file },
  { "grib_index_read", (void *)_cffi_f_grib_index_read, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 51), (void *)_cffi_d_grib_index_read },
  { "grib_index_select_double", (void *)_cffi_f_grib_index_select_double, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 280), (void *)_cffi_d_grib_index_select_double },
  { "grib_index_select_long", (void *)_cffi_f_grib_index_select_long, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 285), (void *)_cffi_d_grib_index_select_long },
  { "grib_index_select_string", (void *)_cffi_f_grib_index_select_string, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 275), (void *)_cffi_d_grib_index_select_string },
  { "grib_index_write", (void *)_cffi_f_grib_index_write, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 271), (void *)_cffi_d_grib_index_write },
  { "grib_is_defined", (void *)_cffi_f_grib_is_defined, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 173), (void *)_cffi_d_grib_is_defined },
  { "grib_is_missing", (void *)_cffi_f_grib_is_missing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 206), (void *)_cffi_d_grib_is_missing },
  { "grib_iterator_delete", (void *)_cffi_f_grib_iterator_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 313), (void *)_cffi_d_grib_iterator_delete },
  { "grib_iterator_new", (void *)_cffi_f_grib_iterator_new, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 56), (void *)_cffi_d_grib_iterator_new },
  { "grib_iterator_next", (void *)_cffi_f_grib_iterator_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 316), (void *)_cffi_d_grib_iterator_next },
  { "grib_keys_iterator_delete", (void *)_cffi_f_grib_keys_iterator_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 322), (void *)_cffi_d_grib_keys_iterator_delete },
  { "grib_keys_iterator_get_name", (void *)_cffi_f_grib_keys_iterator_get_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 10), (void *)_cffi_d_grib_keys_iterator_get_name },
  { "grib_keys_iterator_new", (void *)_cffi_f_grib_keys_iterator_new, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 61), (void *)_cffi_d_grib_keys_iterator_new },
  { "grib_keys_iterator_next", (void *)_cffi_f_grib_keys_iterator_next, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 322), (void *)_cffi_d_grib_keys_iterator_next },
  { "grib_keys_iterator_rewind", (void *)_cffi_f_grib_keys_iterator_rewind, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 322), (void *)_cffi_d_grib_keys_iterator_rewind },
  { "grib_keys_iterator_set_flags", (void *)_cffi_f_grib_keys_iterator_set_flags, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 325), (void *)_cffi_d_grib_keys_iterator_set_flags },
  { "grib_multi_handle_append", (void *)_cffi_f_grib_multi_handle_append, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 168), (void *)_cffi_d_grib_multi_handle_append },
  { "grib_multi_handle_delete", (void *)_cffi_f_grib_multi_handle_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 329), (void *)_cffi_d_grib_multi_handle_delete },
  { "grib_multi_handle_new", (void *)_cffi_f_grib_multi_handle_new, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 66), (void *)_cffi_d_grib_multi_handle_new },
  { "grib_multi_handle_write", (void *)_cffi_f_grib_multi_handle_write, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 332), (void *)_cffi_d_grib_multi_handle_write },
  { "grib_multi_support_off", (void *)_cffi_f_grib_multi_support_off, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_grib_multi_support_off },
  { "grib_multi_support_on", (void *)_cffi_f_grib_multi_support_on, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 358), (void *)_cffi_d_grib_multi_support_on },
  { "grib_multi_support_reset_file", (void *)_cffi_f_grib_multi_support_reset_file, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 361), (void *)_cffi_d_grib_multi_support_reset_file },
  { "grib_nearest_delete", (void *)_cffi_f_grib_nearest_delete, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 336), (void *)_cffi_d_grib_nearest_delete },
  { "grib_nearest_find", (void *)_cffi_f_grib_nearest_find, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 339), (void *)_cffi_d_grib_nearest_find },
  { "grib_nearest_find_multiple", (void *)_cffi_f_grib_nearest_find_multiple, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 246), (void *)_cffi_d_grib_nearest_find_multiple },
  { "grib_nearest_new", (void *)_cffi_f_grib_nearest_new, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 69), (void *)_cffi_d_grib_nearest_new },
  { "grib_samples_path", (void *)_cffi_f_grib_samples_path, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_grib_samples_path },
  { "grib_set_double", (void *)_cffi_f_grib_set_double, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 138), (void *)_cffi_d_grib_set_double },
  { "grib_set_double_array", (void *)_cffi_f_grib_set_double_array, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 132), (void *)_cffi_d_grib_set_double_array },
  { "grib_set_long", (void *)_cffi_f_grib_set_long, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 154), (void *)_cffi_d_grib_set_long },
  { "grib_set_long_array", (void *)_cffi_f_grib_set_long_array, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 148), (void *)_cffi_d_grib_set_long_array },
  { "grib_set_missing", (void *)_cffi_f_grib_set_missing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 116), (void *)_cffi_d_grib_set_missing },
  { "grib_set_string", (void *)_cffi_f_grib_set_string, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 126), (void *)_cffi_d_grib_set_string },
  { "grib_set_string_array", (void *)_cffi_f_grib_set_string_array, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 120), (void *)_cffi_d_grib_set_string_array },
  { "grib_set_values", (void *)_cffi_f_grib_set_values, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 163), (void *)_cffi_d_grib_set_values },
  { "parse_keyval_string", (void *)_cffi_f_parse_keyval_string, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 76), (void *)_cffi_d_parse_keyval_string },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "message_offset", offsetof(codes_bufr_header, message_offset),
                      sizeof(((codes_bufr_header *)0)->message_offset),
                      _CFFI_OP(_CFFI_OP_NOOP, 2) },
  { "message_size", offsetof(codes_bufr_header, message_size),
                    sizeof(((codes_bufr_header *)0)->message_size),
                    _CFFI_OP(_CFFI_OP_NOOP, 2) },
  { "edition", offsetof(codes_bufr_header, edition),
               sizeof(((codes_bufr_header *)0)->edition),
               _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "masterTableNumber", offsetof(codes_bufr_header, masterTableNumber),
                         sizeof(((codes_bufr_header *)0)->masterTableNumber),
                         _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "bufrHeaderSubCentre", offsetof(codes_bufr_header, bufrHeaderSubCentre),
                           sizeof(((codes_bufr_header *)0)->bufrHeaderSubCentre),
                           _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "bufrHeaderCentre", offsetof(codes_bufr_header, bufrHeaderCentre),
                        sizeof(((codes_bufr_header *)0)->bufrHeaderCentre),
                        _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "updateSequenceNumber", offsetof(codes_bufr_header, updateSequenceNumber),
                            sizeof(((codes_bufr_header *)0)->updateSequenceNumber),
                            _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "dataCategory", offsetof(codes_bufr_header, dataCategory),
                    sizeof(((codes_bufr_header *)0)->dataCategory),
                    _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "dataSubCategory", offsetof(codes_bufr_header, dataSubCategory),
                       sizeof(((codes_bufr_header *)0)->dataSubCategory),
                       _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "masterTablesVersionNumber", offsetof(codes_bufr_header, masterTablesVersionNumber),
                                 sizeof(((codes_bufr_header *)0)->masterTablesVersionNumber),
                                 _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localTablesVersionNumber", offsetof(codes_bufr_header, localTablesVersionNumber),
                                sizeof(((codes_bufr_header *)0)->localTablesVersionNumber),
                                _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalYear", offsetof(codes_bufr_header, typicalYear),
                   sizeof(((codes_bufr_header *)0)->typicalYear),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalMonth", offsetof(codes_bufr_header, typicalMonth),
                    sizeof(((codes_bufr_header *)0)->typicalMonth),
                    _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalDay", offsetof(codes_bufr_header, typicalDay),
                  sizeof(((codes_bufr_header *)0)->typicalDay),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalHour", offsetof(codes_bufr_header, typicalHour),
                   sizeof(((codes_bufr_header *)0)->typicalHour),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalMinute", offsetof(codes_bufr_header, typicalMinute),
                     sizeof(((codes_bufr_header *)0)->typicalMinute),
                     _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalSecond", offsetof(codes_bufr_header, typicalSecond),
                     sizeof(((codes_bufr_header *)0)->typicalSecond),
                     _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalDate", offsetof(codes_bufr_header, typicalDate),
                   sizeof(((codes_bufr_header *)0)->typicalDate),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "typicalTime", offsetof(codes_bufr_header, typicalTime),
                   sizeof(((codes_bufr_header *)0)->typicalTime),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "internationalDataSubCategory", offsetof(codes_bufr_header, internationalDataSubCategory),
                                    sizeof(((codes_bufr_header *)0)->internationalDataSubCategory),
                                    _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localSectionPresent", offsetof(codes_bufr_header, localSectionPresent),
                           sizeof(((codes_bufr_header *)0)->localSectionPresent),
                           _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "ecmwfLocalSectionPresent", offsetof(codes_bufr_header, ecmwfLocalSectionPresent),
                                sizeof(((codes_bufr_header *)0)->ecmwfLocalSectionPresent),
                                _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rdbType", offsetof(codes_bufr_header, rdbType),
               sizeof(((codes_bufr_header *)0)->rdbType),
               _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "oldSubtype", offsetof(codes_bufr_header, oldSubtype),
                  sizeof(((codes_bufr_header *)0)->oldSubtype),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rdbSubtype", offsetof(codes_bufr_header, rdbSubtype),
                  sizeof(((codes_bufr_header *)0)->rdbSubtype),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "ident", offsetof(codes_bufr_header, ident),
             sizeof(((codes_bufr_header *)0)->ident),
             _CFFI_OP(_CFFI_OP_NOOP, 382) },
  { "localYear", offsetof(codes_bufr_header, localYear),
                 sizeof(((codes_bufr_header *)0)->localYear),
                 _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localMonth", offsetof(codes_bufr_header, localMonth),
                  sizeof(((codes_bufr_header *)0)->localMonth),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localDay", offsetof(codes_bufr_header, localDay),
                sizeof(((codes_bufr_header *)0)->localDay),
                _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localHour", offsetof(codes_bufr_header, localHour),
                 sizeof(((codes_bufr_header *)0)->localHour),
                 _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localMinute", offsetof(codes_bufr_header, localMinute),
                   sizeof(((codes_bufr_header *)0)->localMinute),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localSecond", offsetof(codes_bufr_header, localSecond),
                   sizeof(((codes_bufr_header *)0)->localSecond),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rdbtimeDay", offsetof(codes_bufr_header, rdbtimeDay),
                  sizeof(((codes_bufr_header *)0)->rdbtimeDay),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rdbtimeHour", offsetof(codes_bufr_header, rdbtimeHour),
                   sizeof(((codes_bufr_header *)0)->rdbtimeHour),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rdbtimeMinute", offsetof(codes_bufr_header, rdbtimeMinute),
                     sizeof(((codes_bufr_header *)0)->rdbtimeMinute),
                     _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rdbtimeSecond", offsetof(codes_bufr_header, rdbtimeSecond),
                     sizeof(((codes_bufr_header *)0)->rdbtimeSecond),
                     _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rectimeDay", offsetof(codes_bufr_header, rectimeDay),
                  sizeof(((codes_bufr_header *)0)->rectimeDay),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rectimeHour", offsetof(codes_bufr_header, rectimeHour),
                   sizeof(((codes_bufr_header *)0)->rectimeHour),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rectimeMinute", offsetof(codes_bufr_header, rectimeMinute),
                     sizeof(((codes_bufr_header *)0)->rectimeMinute),
                     _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "rectimeSecond", offsetof(codes_bufr_header, rectimeSecond),
                     sizeof(((codes_bufr_header *)0)->rectimeSecond),
                     _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "restricted", offsetof(codes_bufr_header, restricted),
                  sizeof(((codes_bufr_header *)0)->restricted),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "isSatellite", offsetof(codes_bufr_header, isSatellite),
                   sizeof(((codes_bufr_header *)0)->isSatellite),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "localLongitude1", offsetof(codes_bufr_header, localLongitude1),
                       sizeof(((codes_bufr_header *)0)->localLongitude1),
                       _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "localLatitude1", offsetof(codes_bufr_header, localLatitude1),
                      sizeof(((codes_bufr_header *)0)->localLatitude1),
                      _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "localLongitude2", offsetof(codes_bufr_header, localLongitude2),
                       sizeof(((codes_bufr_header *)0)->localLongitude2),
                       _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "localLatitude2", offsetof(codes_bufr_header, localLatitude2),
                      sizeof(((codes_bufr_header *)0)->localLatitude2),
                      _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "localLatitude", offsetof(codes_bufr_header, localLatitude),
                     sizeof(((codes_bufr_header *)0)->localLatitude),
                     _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "localLongitude", offsetof(codes_bufr_header, localLongitude),
                      sizeof(((codes_bufr_header *)0)->localLongitude),
                      _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "localNumberOfObservations", offsetof(codes_bufr_header, localNumberOfObservations),
                                 sizeof(((codes_bufr_header *)0)->localNumberOfObservations),
                                 _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "satelliteID", offsetof(codes_bufr_header, satelliteID),
                   sizeof(((codes_bufr_header *)0)->satelliteID),
                   _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "qualityControl", offsetof(codes_bufr_header, qualityControl),
                      sizeof(((codes_bufr_header *)0)->qualityControl),
                      _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "newSubtype", offsetof(codes_bufr_header, newSubtype),
                  sizeof(((codes_bufr_header *)0)->newSubtype),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "daLoop", offsetof(codes_bufr_header, daLoop),
              sizeof(((codes_bufr_header *)0)->daLoop),
              _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "numberOfSubsets", offsetof(codes_bufr_header, numberOfSubsets),
                       sizeof(((codes_bufr_header *)0)->numberOfSubsets),
                       _CFFI_OP(_CFFI_OP_NOOP, 2) },
  { "observedData", offsetof(codes_bufr_header, observedData),
                    sizeof(((codes_bufr_header *)0)->observedData),
                    _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "compressedData", offsetof(codes_bufr_header, compressedData),
                      sizeof(((codes_bufr_header *)0)->compressedData),
                      _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "name", offsetof(grib_values, name),
            sizeof(((grib_values *)0)->name),
            _CFFI_OP(_CFFI_OP_NOOP, 31) },
  { "type", offsetof(grib_values, type),
            sizeof(((grib_values *)0)->type),
            _CFFI_OP(_CFFI_OP_NOOP, 14) },
  { "long_value", offsetof(grib_values, long_value),
                  sizeof(((grib_values *)0)->long_value),
                  _CFFI_OP(_CFFI_OP_NOOP, 157) },
  { "double_value", offsetof(grib_values, double_value),
                    sizeof(((grib_values *)0)->double_value),
                    _CFFI_OP(_CFFI_OP_NOOP, 141) },
  { "string_value", offsetof(grib_values, string_value),
                    sizeof(((grib_values *)0)->string_value),
                    _CFFI_OP(_CFFI_OP_NOOP, 31) },
  { "error", offsetof(grib_values, error),
             sizeof(((grib_values *)0)->error),
             _CFFI_OP(_CFFI_OP_NOOP, 14) },
  { "has_value", offsetof(grib_values, has_value),
                 sizeof(((grib_values *)0)->has_value),
                 _CFFI_OP(_CFFI_OP_NOOP, 14) },
  { "equal", offsetof(grib_values, equal),
             sizeof(((grib_values *)0)->equal),
             _CFFI_OP(_CFFI_OP_NOOP, 14) },
  { "next", offsetof(grib_values, next),
            sizeof(((grib_values *)0)->next),
            _CFFI_OP(_CFFI_OP_NOOP, 81) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "_IO_FILE", 379, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "bufr_keys_iterator", 380, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "codes_bufr_header", 385, _CFFI_F_CHECK_FIELDS,
    sizeof(codes_bufr_header), offsetof(struct _cffi_align__codes_bufr_header, y), 0, 56 },
  { "grib_context", 387, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "grib_handle", 388, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "grib_index", 389, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "grib_iterator", 390, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "grib_keys_iterator", 391, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "grib_multi_handle", 392, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "grib_nearest", 393, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "grib_values", 394, _CFFI_F_CHECK_FIELDS,
    sizeof(grib_values), offsetof(struct _cffi_align__grib_values, y), 56, 9 },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "ProductKind", 21, _cffi_prim_int(sizeof(ProductKind), ((ProductKind)-1) <= 0),
    "PRODUCT_ANY,PRODUCT_GRIB,PRODUCT_BUFR,PRODUCT_METAR,PRODUCT_GTS,PRODUCT_TAF" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "FILE", 379 },
  { "ProductKind", 21 },
  { "bufr_keys_iterator", 380 },
  { "codes_bufr_header", 385 },
  { "codes_context", 387 },
  { "codes_handle", 388 },
  { "grib_context", 387 },
  { "grib_handle", 388 },
  { "grib_index", 389 },
  { "grib_iterator", 390 },
  { "grib_keys_iterator", 391 },
  { "grib_multi_handle", 392 },
  { "grib_nearest", 393 },
  { "grib_values", 394 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  188,  /* num_globals */
  11,  /* num_struct_unions */
  1,  /* num_enums */
  14,  /* num_typenames */
  NULL,  /* no includes */
  396,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__bindings(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__bindings(void) { return NULL; }
#  else
     init_bindings(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__bindings(void)
{
  return _cffi_init("gribapi._bindings", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_bindings(void)
{
  _cffi_init("gribapi._bindings", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
